import { z } from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import { useForm } from "react-hook-form";

import { Icon } from "shared/assets/icons";
import {
  MODAL_NAMES,
  MODAL_TYPES,
  useModalStore,
} from "shared/stores/modal-store";
import { Box, Button, IconButton, Tooltip } from "shared/ui/components";
import { Form } from "shared/ui/form";
import { FormFieldSelect, FormFieldText } from "shared/ui/form/Input";
import { CustomModal, PrimarySecondaryText } from "shared/ui/templates";
import { useHall, useUpdateHall } from "shared/queries/halls";
import { useEffect } from "react";

const schema = z.object({
  hallNumber: z.string(),
  time: z.number().optional(),
  note: z.string(),
});

type FormData = z.infer<typeof schema>;

export function EditHallCards() {
  const store = useModalStore();
  // const [open, setOpen] = useState(false);
  // const [generated, setGenerated] = useState("");
  const updateHall = useUpdateHall();
  const hall = useHall(store.modalData.id);

  const { control, handleSubmit, reset } = useForm<FormData>({
    defaultValues: {
      hallNumber: "",
      time: 0,
      note: "",
    },
    resolver: zodResolver(schema),
  });

  async function save(data: FormData) {
    const { hallNumber, note, time } = data;
    await updateHall.mutateAsync({
      id: String(store.modalData.id),
      name: hallNumber,
      sleepTime: Number(time),
      notes: note,
    });
    store.actions.closeModal();
  }

  // function onGenerate() {
  //   console.log("onGenerate");
  //   setGenerated(String(getRandomIntInclusive(1000, 9999)));
  //   handleClose();
  // }

  // function handleClose() {
  //   setOpen(false);
  // }

  useEffect(() => {
    if (!hall.data) return;

    reset({
      hallNumber: hall.data.hallName,
      time: hall.data.sleepTime,
      note: hall.data.notes,
    });
  }, [hall.data]);

  return (
    <CustomModal
      title={MODAL_NAMES.EditHallCards}
      open={!!store.openModalType}
      handleClose={() => store.actions.closeModal()}
    >
      <PrimarySecondaryText
        secondary="ID"
        primary={store.modalData.id}
        variant="firstSecondary"
      />
      <PrimarySecondaryText
        secondary="Телевизор"
        primary={store.modalData.tv}
        variant="firstSecondary"
      />
      <PrimarySecondaryText
        secondary="Планшет"
        primary={store.modalData.tablet}
        variant="firstSecondary"
      />
      <Form onSubmit={handleSubmit(save)}>
        <FormFieldText
          control={control}
          name="hallNumber"
          label="Номер зала"
          required
        />
        <FormFieldSelect
          actions
          control={control}
          name="time"
          label="Время до блокировки экрана планшета"
          options={TIMES}
        />
        <FormFieldText control={control} name="note" label="Примечание" />
        {/* TODO: раскомментить после MVP */}
        {/* <Button
          variant="outlined"
          color="primary"
          onClick={() => setOpen(true)}
        >
          Сгенерировать PIN-код
        </Button> */}
        <PrimarySecondaryText
          primary={hall.data?.pin || "-"}
          secondary="PIN-код"
          variant="firstSecondary"
        />
        <Box sx={{ display: "flex", alignItems: "center", gap: 2 }}>
          <Button color="primary" type="submit">
            Сохранить
          </Button>
          <Button color="secondary" onClick={() => store.actions.closeModal()}>
            Отмена
          </Button>
        </Box>
      </Form>

      {/* <CustomModal
        open={open}
        handleClose={handleClose}
        title="Генерация PIN-кода"
        actions={
          <>
            <Button color="primary" onClick={onGenerate}>
              Сгенерировать
            </Button>
            <Button onClick={handleClose} color="secondary">
              Отмена
            </Button>
          </>
        }
      >
        <Typography variant="body2">
          Вы уверены, что хотите сгенерировать новый PIN-код для зала{" "}
          {store.modalData.numberHall}?
        </Typography>
        <Typography variant="body2">
          Это приведет к завершению сессии на устройстве зала и сбросу старого
          PIN-кода.
        </Typography>
      </CustomModal> */}
    </CustomModal>
  );
}

type IconButtonEditHallCards = {
  show: boolean;
  data: any;
};
export function IconButtonEditHallCards(props: IconButtonEditHallCards) {
  const { data, show } = props;
  const store = useModalStore();

  if (!show) return null;

  return (
    <Tooltip title="Редактировать">
      <IconButton
        onClick={() => store.actions.showModal(MODAL_TYPES.EditHallCards, data)}
      >
        <Icon.EditIcon />
      </IconButton>
    </Tooltip>
  );
}

const TIMES = [
  { label: "Без блокировки", value: 0 },
  { label: "30 минут", value: 30 },
  { label: "1 час", value: 60 },
  { label: "2 час", value: 60 * 2 },
  { label: "3 час", value: 60 * 3 },
  { label: "4 час", value: 60 * 4 },
  { label: "5 час", value: 60 * 5 },
  { label: "6 час", value: 60 * 6 },
  { label: "7 час", value: 60 * 7 },
  { label: "8 час", value: 60 * 8 },
];

// function getRandomIntInclusive(min: number, max: number) {
//   const minCeiled = Math.ceil(min);
//   const maxFloored = Math.floor(max);
//   return Math.floor(Math.random() * (maxFloored - minCeiled + 1) + minCeiled); // The maximum is inclusive and the minimum is inclusive
// }
