import { Box, Table, Tooltip, Typography } from "shared/ui/components";
import { Icon } from "shared/assets/icons";
import {
  useTableSorting,
  withProviderTableSorting,
} from "shared/stores/table-sorting";
import { usePagination } from "shared/stores/pagination";
import {
  JUDGE_STATUS_COLORS,
  JUDGE_STATUS_NAMES,
  JUDGE_STATUS_TYPES,
} from "shared/constants/judge";
import { JudgeEntity } from "shared/api";

import { IconButtonDeleteAccount } from "features/delete-account";
import { IconButtonAccountBlocking } from "features/account-blocking";
import { IconButtonUnlockAccount } from "features/unlock-account";
import { IconButtonEndTheSession } from "features/end-the-session";
import { IconButtonEditAccount } from "features/edit-account";
import { useJudgeListQuery } from "shared/queries/judges";

function JudgesTableComponent() {
  const tableSorting = useTableSorting();
  const pagination = usePagination();

  const judges = useJudgeListQuery({
    page: pagination.page,
    pageSize: pagination.pageSize,
    order: tableSorting.sortingString,
  });

  return (
    <Table
      loading={judges.isLoading}
      columns={COLUMNS}
      rows={judges.data?.items ?? []}
      pagination={{
        page: pagination.page,
        pageSize: pagination.pageSize,
        rowCount: judges.data?.rowCount ?? 0,
        onPageChange: pagination.updatePage,
        onPageSizeChange: pagination.updatePageSize,
      }}
      sorting={{
        onOrederChange: ({ name, value }) =>
          tableSorting.handleSortingChange(name, value),
        order: tableSorting.order,
        orderBy: tableSorting.key,
      }}
    />
  );
}

export const JudgesTable = withProviderTableSorting(JudgesTableComponent);

export const COLUMNS = [
  { field: "fio", headerName: "ФИО" },
  { field: "email", headerName: "E-mail" },
  {
    field: "status",
    renderCell: (row: JudgeEntity) => (
      <Box sx={{ display: "flex", alignItems: "center", gap: 1 }}>
        <Typography color={JUDGE_STATUS_COLORS[row?.status]} variant="body2">
          {JUDGE_STATUS_NAMES[row?.status]}
        </Typography>
        {row?.description && (
          <Tooltip title={row?.description || "TEST"} placement="top-start">
            <Icon.Info />
          </Tooltip>
        )}
      </Box>
    ),
    headerName: "Статус учетной записи",
  },
  {
    field: "actions",
    renderCell: (row: any) => {
      const isActive = row?.status === JUDGE_STATUS_TYPES.Active;
      const isBlocked = row?.status === JUDGE_STATUS_TYPES.Blocked;

      return (
        <Box
          sx={{
            display: "flex",
            alignItems: "center",
            justifyContent: "end",
          }}
        >
          <IconButtonEditAccount data={row} show={true} />

          {isActive && <IconButtonAccountBlocking data={row} show={true} />}

          {isBlocked && <IconButtonUnlockAccount data={row} show={true} />}

          <IconButtonEndTheSession data={row} show={true} />

          <IconButtonDeleteAccount data={row} show={true} />
        </Box>
      );
    },
  },
];
