import { DEVICE_TYPE_ID } from "shared/constants/device";
import { HallsRepositories } from "shared/repositories/HallsRepositories";
import {
  HallCardResponseDTO,
  HallChangeStatusParams,
  HallEditRequestParams,
  HallElementDTO,
  HallsByDeviceType,
  HallsList,
  HallsParams,
} from "shared/types";
import { HallEntity } from "shared/types/halls/entities";

class HallsService {
  static #instance: HallsService;
  #repo: HallsRepositories;

  private constructor() {
    this.#repo = new HallsRepositories();
  }

  public static get instance(): HallsService {
    if (!HallsService.#instance) {
      HallsService.#instance = new HallsService();
    }

    return HallsService.#instance;
  }

  private HallDtoToHallEntity(dto: HallElementDTO): HallEntity {
    const { blockReason, devices, hallId, hallName, statusId, notes } = dto;
    const { tablet, tv } = devices?.reduce<{ tablet: string[]; tv: string[] }>(
      (acc, device) => {
        if (device.deviceType === DEVICE_TYPE_ID.Tablets) {
          acc.tablet.push(String(device.id));
        }
        if (device.deviceType === DEVICE_TYPE_ID.SamllTv) {
          acc.tv.push(String(device.id));
        }
        if (device.deviceType === DEVICE_TYPE_ID.BigTv) {
          acc.tv.push(String(device.id));
        }

        return acc;
      },
      { tablet: [], tv: [] },
    );

    return {
      id: String(hallId),
      hallNumber: String(hallName),
      note: notes,
      reasonBlocking: String(blockReason).trim(),
      statusId: Number(statusId),
      tablet: tablet.join(" "),
      tv: tv.join(" "),
    };
  }

  async getHallsList(params: HallsParams): Promise<HallsList> {
    const data = await this.#repo.getHalls(params);
    const items = data.content.map(this.HallDtoToHallEntity);

    return { items, rowCount: data.totalElements };
  }

  async getHallsByDeviceType(
    deviceTypeId: number,
  ): Promise<HallsByDeviceType[]> {
    const data = await this.#repo.getHallsByDeviceId({
      deviceTypeId: deviceTypeId,
    });

    return data.halls.map((item) => ({
      value: item.hallId,
      label: item.hallName,
    }));
  }

  async getCardById(id: string): Promise<HallCardResponseDTO> {
    const data = await this.#repo.getCardById(id);

    return data;
  }

  updateHall(params: HallEditRequestParams): Promise<void> {
    return this.#repo.updateHall(params);
  }

  changeStatus(params: HallChangeStatusParams): Promise<void> {
    return this.#repo.changeStatus(params);
  }
}

export const hallsService = HallsService.instance;
