import { create } from "zustand";
import { devtools } from "zustand/middleware";
import { immer } from "zustand/middleware/immer";

import { TableProps } from "shared/ui/components";

import { FieldQueryFilterProps } from "shared/ui/templates";
import { PaginationResponseDto } from "shared/api/common";

type Table = { rows: TableProps["rows"]; rowCount: number };
type Pagination = { page: number; pageSize: number };
type Query = { [key: string]: any };
type State = {
  loading: boolean;
  table: Table;
  pagination: Pagination;
  query: Query;
  queryFilterFields: FieldQueryFilterProps[];
  order: "asc" | "desc";
  orderBy: string;
};
type Actions = {
  setLoading: (value: boolean) => void;
  setTable: (value: Table) => void;
  setPagination: (value: Pagination) => void;
  onPageChange: (value: number) => void;
  onPageSizeChange: (value: number) => void;
  setQuery: (value: Query) => void;
  changeQuery: (value: { name: string; value: any }) => void;
  setQueryFilterOptionsByName: (name: string, options: Option[]) => void;
  setQueryFilter: (value: FieldQueryFilterProps[]) => void;
  getTableData: () => Promise<void>;
  onOrederChange: ({
    name,
    value,
  }: {
    name: string;
    value: "asc" | "desc";
  }) => void;
  init: () => void;
  reset: () => void;
};
type Store = State & { actions: Actions };
type Option = { value: string; label?: string };
type QueryFilterField = {
  type: string;
  label: string;
  name: string;
  defaultValue: unknown;
  options?: Option[];
  getOptions?: () => Promise<Option[]>;
};
type CreatePageStoreParams = {
  queryFilterConfig?: QueryFilterField[];
  tableConfig: {
    getData: (query: any) => Promise<PaginationResponseDto<any>>;
  };
  sorting?: {
    order?: "asc" | "desc";
    orderBy?: string;
  };
};
export function createPageStore(params: CreatePageStoreParams) {
  const { queryFilterConfig = [], tableConfig, sorting } = params;

  const query = queryFilterConfig.reduce<Query>((acc, item) => {
    acc[item.name] = item.defaultValue;
    return acc;
  }, {});

  const queryFilterFields: FieldQueryFilterProps[] = queryFilterConfig.map(
    (item) => {
      const { label, name, type, options } = item;
      return { label, name, type, options };
    },
  );

  const INITIAL_STATE: State = {
    loading: false,
    table: { rowCount: 0, rows: [] },
    pagination: { page: 1, pageSize: 10 },
    queryFilterFields: [...queryFilterFields],
    query: { ...query },
    order: sorting?.order ?? "asc",
    orderBy: sorting?.orderBy ?? "status",
  };

  const useStore = create<Store>()(
    devtools(
      immer((set, get) => ({
        ...INITIAL_STATE,
        actions: {
          setLoading: (value) => {
            set((state) => void (state.loading = value));
          },
          setTable: (value) => {
            set((state) => void (state.table = value));
          },
          setPagination: (value) => {
            set((state) => void (state.pagination = value));
          },
          onPageChange: (value: number) => {
            set((state) => void (state.pagination.page = value));
            get().actions.getTableData();
          },
          onPageSizeChange: (value: number) => {
            set((state) => void (state.pagination.pageSize = value));
            get().actions.getTableData();
          },
          setQuery: (value) => {
            set((state) => void (state.query = { ...value }));
          },
          changeQuery: ({ name, value }) => {
            set((state) => {
              state.query = { ...state.query, [name]: value };
            });
            get().actions.getTableData();
          },
          setQueryFilter: (value) => {
            set((state) => void (state.queryFilterFields = value));
          },
          setQueryFilterOptionsByName: (name: string, options: Option[]) => {
            set((state) => {
              const field = state.queryFilterFields.find(
                (i) => i.name === name,
              );
              if (!field?.options) return;
              field.options = options;
            });
          },
          getTableData: async () => {
            const { actions, query, pagination } = get();
            actions.setLoading(true);
            const { rowCount, rows } = await tableConfig.getData({
              ...query,
              ...pagination,
            });
            actions.setTable({ rowCount, rows });
            actions.setLoading(false);
          },
          onOrederChange: ({ name, value }) => {
            set((state) => {
              state.order = value;
              state.orderBy = name;
            });
          },
          init: async () => {
            const { actions } = get();

            actions.getTableData();

            queryFilterConfig.forEach(async (i) => {
              if (!i.getOptions) return;
              const options = await i.getOptions();
              actions.setQueryFilterOptionsByName(i.name, options);
            });
          },
          reset: () => {
            const { actions } = get();
            actions.setPagination(INITIAL_STATE.pagination);
            actions.setQueryFilter(INITIAL_STATE.queryFilterFields);
            actions.setQuery(INITIAL_STATE.query);
            actions.setTable(INITIAL_STATE.table);
          },
        },
      })),
    ),
  );

  return { useStore };
}
