import {
  ComponentType,
  Dispatch,
  ReactNode,
  SetStateAction,
  createContext,
  useContext,
  useMemo,
  useState,
} from "react";

type Order = "asc" | "desc" | null;

const ORDER_TYPE = {
  ASC: "asc",
  DESC: "desc",
  NULL: null,
} as const;

export type TableSortingData = {
  order: Order;
  key: string | null;
};

const initialState: TableSortingData = { order: null, key: null };

const tableSortingContext = createContext<{
  data: TableSortingData;
  setData: Dispatch<SetStateAction<TableSortingData>>;
} | null>(null);

export function TableSortingProvider({ children }: { children: ReactNode }) {
  const [data, setData] = useState<TableSortingData>(initialState);

  return (
    <tableSortingContext.Provider
      value={useMemo(() => ({ data, setData }), [data])}
    >
      {children}
    </tableSortingContext.Provider>
  );
}

function getNextOrder(order: Order): Order {
  switch (order) {
    case ORDER_TYPE.ASC:
      return ORDER_TYPE.DESC;
    case ORDER_TYPE.DESC:
      return ORDER_TYPE.NULL;
    case ORDER_TYPE.NULL:
      return ORDER_TYPE.ASC;
  }
}

export function useTableSorting() {
  const contextValue = useContext(tableSortingContext);
  if (!contextValue) throw new Error("context not provided");

  const { data, setData } = contextValue;

  const handleSortingChange = (key: string, order: Order) => {
    if (key !== data.key) {
      setData({ key, order: ORDER_TYPE.ASC });
      return;
    }
    setData((d) => ({ ...d, order: getNextOrder(order) }));
  };

  const sortingString = useMemo(() => {
    if (!data.order) return undefined;
    return `${data.key}=${data.order}`;
  }, [data]);

  const reset = () => {
    setData(initialState);
  };

  return {
    ...data,
    ...(data.order && { asc: `${data.order === "asc"}` }),
    sortingString,
    handleSortingChange,
    reset,
  };
}

export function withProviderTableSorting<P extends object>(
  WrappedComponent: ComponentType<P>,
): React.FC<P> {
  const WithTableSorting: React.FC<P> = ({ ...props }: P) => {
    return (
      <TableSortingProvider>
        <WrappedComponent {...(props as P)} />
      </TableSortingProvider>
    );
  };

  return WithTableSorting;
}
