import {
  FormControl,
  FormControlProps,
  FormHelperText,
  InputLabel,
} from "@mui/material";
import MuiSelect, { SelectChangeEvent } from "@mui/material/Select";
import MenuItem from "@mui/material/MenuItem";
import { useMemo } from "react";

type Option = {
  value: string | number | null | undefined;
  label?: string;
  iconStart?: React.ReactNode;
  iconEnd?: React.ReactNode;
};

export type SelectProps = {
  value: string;
  onChange?: (event: any) => void;
  name?: string;
  options?: Option[];
  label?: string;
  helperText?: string | null;
  error?: boolean;
  renderValue?: (value: string) => string;
};
export function Select(props: SelectProps & FormControlProps) {
  const {
    label,
    value,
    name,
    onChange,
    options = [],
    helperText,
    error,
    renderValue: customRenderValue,
    required,
    ...formControlProps
  } = props;

  const valueLabelMaps = useMemo(() => {
    return options.reduce<Record<string, string>>((acc, i) => {
      acc[String(i.value)] = i.label ?? "";
      return acc;
    }, {});
  }, [options]);

  const handleChange = (event: SelectChangeEvent<typeof value>) => {
    onChange && onChange(event as any);
  };

  const renderValue = (value: string) => {
    if (!customRenderValue) {
      return valueLabelMaps[value] ?? value;
    }
    return customRenderValue(value);
  };

  return (
    <FormControl
      variant="standard"
      fullWidth
      error={error}
      {...formControlProps}
    >
      <InputLabel required={required} id={name}>
        {label}
      </InputLabel>
      <MuiSelect
        id={name}
        labelId={name}
        value={value}
        label={label ?? name}
        onChange={handleChange}
        renderValue={renderValue}
      >
        {options.map((option) => {
          const { label, value: v, iconEnd, iconStart } = option;
          return (
            <MenuItem key={String(v)} value={v as string}>
              {iconStart}
              {label ?? v}
              {iconEnd}
            </MenuItem>
          );
        })}
      </MuiSelect>
      {helperText && <FormHelperText>{helperText}</FormHelperText>}
    </FormControl>
  );
}
