import { styled } from "@mui/material/styles";
import MuiTable from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell, { tableCellClasses } from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TableHead from "@mui/material/TableHead";
import TableRow, { tableRowClasses } from "@mui/material/TableRow";
import { Pagination } from "@mui/material";
import { Column, PaginationProps, Row, SortingProps } from "./types";
import { Box } from "../Box/Box";
import { useEffect, useRef, useState } from "react";
import { IconButton } from "../IconButton/IconButton";
import { Icon } from "shared/assets/icons";

export type TableProps = {
  rows: Row[];
  columns: Column[];
  loading?: boolean;
  pagination?: PaginationProps;
  sorting?: SortingProps;
};

export function Table(props: TableProps) {
  const { columns, rows, loading, pagination, sorting } = props;

  const boxRef = useRef<any>();
  const [height, setHeight] = useState<string | number>("100%");

  useEffect(() => {
    function handleResize() {
      setHeight(boxRef?.current?.clientHeight - 60);
    }

    handleResize();

    window.addEventListener("resize", handleResize);

    return () => window.removeEventListener("resize", handleResize);
  }, []);

  return (
    <Box
      ref={boxRef}
      sx={{
        display: "flex",
        flexDirection: "column",
        gap: 2,
        flexGrow: 1,
        justifyContent: "space-between",
      }}
    >
      <TableContainer style={{ maxHeight: height }}>
        <MuiTable stickyHeader>
          <TableHead>
            <TableRow>
              {columns.map((column) => {
                const {
                  align,
                  field,
                  headerName,
                  renderHeader,
                  width,
                  enableOrder,
                } = column;

                return (
                  <StyledTableCell key={field} align={align} style={{ width }}>
                    <Box sx={{ display: "flex", gap: 1, alignItems: "center" }}>
                      {renderHeader ? renderHeader() : headerName}
                      {/* sorting.orderBy === field && */}
                      {enableOrder && sorting && (
                        <IconButton
                          onClick={() =>
                            sorting?.onOrederChange({
                              name: field,
                              value: sorting.order,
                            })
                          }
                        >
                          <Icon.ChevronDown
                            sx={{
                              transition: "transform 0.3s",
                              transform: `rotate(${sorting.order === "desc" ? "180" : "0"}deg)`,
                              opacity: sorting.order ? 1 : 0.3,
                            }}
                          />
                        </IconButton>
                      )}
                    </Box>
                  </StyledTableCell>
                );
              })}
            </TableRow>
          </TableHead>
          <TableBody>
            {rows.map((row, rowIndex) => {
              return (
                <StyledTableRow hover key={row.id}>
                  {columns.map((column) => {
                    const { field, align, renderCell, width } = column;
                    const value = row[field];

                    return (
                      <StyledTableCell
                        key={field}
                        scope="row"
                        align={align}
                        className={field === "actions" ? "actions" : ""}
                        style={{ width }}
                      >
                        {(renderCell ? renderCell(row, rowIndex) : value) ??
                          "-"}
                      </StyledTableCell>
                    );
                  })}
                </StyledTableRow>
              );
            })}
          </TableBody>
        </MuiTable>
      </TableContainer>
      {pagination && (
        <Pagination
          disabled={loading}
          count={Math.ceil(pagination.rowCount / pagination.pageSize) || 1}
          page={pagination.page}
          onChange={(_event, newPage) => pagination.onPageChange(newPage)}
        />
      )}
    </Box>
  );
}

const StyledTableCell = styled(TableCell)(({ theme }) => ({
  [`&.${tableCellClasses.head}`]: {
    backgroundColor: theme.colors.BG_LIGHT,
    color: theme.colors.GRAY_LIGHT,
  },
  [`&.${tableCellClasses.body}`]: {
    transition: "opacity 0.3s",
    fontSize: 14,
  },
  [`&.${tableCellClasses.body}.actions`]: {
    opacity: 0,
  },
}));

const StyledTableRow = styled(TableRow)(({ theme }) => ({
  [`&.${tableRowClasses.hover}:hover`]: {
    backgroundColor: theme.colors.BG_HOVER,
  },
  [`&.${tableRowClasses.hover}:hover .${tableCellClasses.body}.actions`]: {
    opacity: 1,
  },
  "&:nth-of-type(odd)": {
    backgroundColor: theme.colors.BG_DARK,
  },
  // hide last border
  "&:last-child td, &:last-child th": {
    border: 0,
  },
}));
