package ru.avilex.session.service;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import ru.avilex.session.dto.ChangeRequestHallIdDTO;
import ru.avilex.session.dto.GeneratePinResponseDTO;
import ru.avilex.session.dto.LogoutRequestDTO;
import ru.avilex.session.dto.request.HallEditRequestDTO;
import ru.avilex.session.enumeration.DeviceHallStatus;
import ru.avilex.session.enumeration.DeviceType;
import ru.avilex.session.exception.*;
import ru.avilex.session.model.Device;
import ru.avilex.session.model.DeviceHall;
import ru.avilex.session.repository.DeviceHallRepository;
import ru.avilex.session.repository.DeviceRepository;
import ru.avilex.session.validator.DeviceConfigurationValidator;

import java.util.List;
import java.util.Optional;
import java.util.UUID;


@Service
@RequiredArgsConstructor
public class DeviceHallService {

    private final DeviceHallRepository deviceHallRepository;
    private final DeviceRepository deviceRepository;
    private final DeviceConfigurationValidator deviceConfigurationValidator;

    @Transactional
    public void save(DeviceHall deviceHall) {
        deviceHallRepository.save(deviceHall);
    }

    @Transactional
    public void changeStatus(UUID deviceHallId, DeviceHallStatus deviceHallStatus) {
        DeviceHall deviceHall = getDeviceHall(deviceHallId);
        List<Device> deviceList = deviceRepository.getListByHallId(deviceHallId, DeviceType.TABLET.getId());
        deviceList.forEach(deviceConfigurationValidator::validateOnDeviceHallBlockedOrDeleted);
        deviceHall.setStatus(deviceHallStatus.getCode());
        try {
            deviceHallRepository.save(deviceHall);
            deviceList.forEach(deviceRepository::save);
        } catch (Exception ex) {
            throw new DeviceHallChangeStatusException();
        }
    }

    @Transactional
    public void changeStatusLogout(LogoutRequestDTO requestDTO) {
        DeviceHall deviceHall = getDeviceHall(requestDTO.getHallId());
        if (deviceHall.getStatus() == DeviceHallStatus.DELETED.getCode()) {
            throw new DeviceHallChangeStatusDeleteException();
        }
        deviceHall.setStatus(DeviceHallStatus.LOGOUT.getCode());
        deviceHallRepository.save(deviceHall);

    }


    @Transactional
    public void changeStatusBlock(UUID deviceHallId, DeviceHallStatus deviceHallStatus, String blockReason) {
        DeviceHall deviceHall = getDeviceHall(deviceHallId);
        if (deviceHall.getStatus() == DeviceHallStatus.BLOCKED_BY_PIN.getCode()
                && deviceHallStatus.getCode() == DeviceHallStatus.LOGOUT.getCode()) {
            deviceHall.setFailedAttempts((short) 0);
            deviceHall.setPin(PinCodeService.generateRandomPin());
        }
        deviceHall.setStatus(deviceHallStatus.getCode());
        if (deviceHall.getStatus() == DeviceHallStatus.DELETED.getCode()) {
            throw new DeviceHallBlockedOrDeletedException();
        }
        if (deviceHallStatus.getCode() == DeviceHallStatus.ACTIVATED.getCode()) {
            throw new DeviceHallBlockCardException();
        }
        deviceHall.setBlockReason(deviceHall.getStatus() == DeviceHallStatus.BLOCKED.getCode() ? blockReason : "");
        try {
            deviceHallRepository.save(deviceHall);
        } catch (Exception ex) {
            throw new DeviceHallChangeStatusException();
        }
    }

    @Transactional
    public GeneratePinResponseDTO changePin(ChangeRequestHallIdDTO changeRequestHallIdDTO) {
        List<Device> deviceList = deviceRepository.getListByHallId(changeRequestHallIdDTO.getHallId(), DeviceType.TABLET.getId());
        if (deviceList.isEmpty()) {
            throw new NoTabletDeviceException();
        }
        String pin = PinCodeService.generateRandomPin();
        deviceList.forEach(device -> {
            device.getDeviceHall().setPin(pin);
            deviceRepository.save(device);
        });
        return new GeneratePinResponseDTO(pin);
    }

    @Transactional
    public void update(HallEditRequestDTO requestDTO) {
        DeviceHall deviceHall = getDeviceHall(requestDTO.getId());
        List<Device> deviceList = deviceRepository.getListByHallId(requestDTO.getId(), DeviceType.TABLET.getId());
        deviceList.forEach(deviceConfigurationValidator::validateOnDeviceHallBlockedOrDeleted);
        if (!deviceList.isEmpty()) {
            deviceHall.setName(requestDTO.getName());
            deviceHall.setNotes(requestDTO.getNotes());
            deviceHall.setSleepTime(requestDTO.getSleepTime());
            deviceHallRepository.save(deviceHall);
        } else {
            throw new ImpossibleEnterNewValueException();
        }
    }

    private DeviceHall getDeviceHall(UUID deviceHallId) {
        return Optional.of(deviceHallRepository.getReferenceById(deviceHallId))
                .orElseThrow(() -> new EntityNotFoundException(String.format("Device hall not found by given id: %s", deviceHallId)));
    }

}


