package ru.avilex.session.service;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.avilex.session.config.WebSocketPropertiesConfig;
import ru.avilex.session.dto.BigTvUpdatedConfigMessageDTO;
import ru.avilex.session.dto.BigTvUpdatedListMessageDTO;
import ru.avilex.session.dto.request.BigTvDeviceUpdateRequestDTO;
import ru.avilex.session.dto.request.DeviceUpdateRequestDTO;
import ru.avilex.session.dto.response.PinCodeAuthResponseDTO;
import ru.avilex.session.enumeration.DeviceStatus;
import ru.avilex.session.enumeration.DeviceType;
import ru.avilex.session.exception.BusinessException;
import ru.avilex.session.exception.IdentificatorValidationException;
import ru.avilex.session.exception.DeviceHallChangeStatusException;
import ru.avilex.session.exception.EntityNotFoundException;
import ru.avilex.session.mapper.ResponseMapper;
import ru.avilex.session.model.Device;
import ru.avilex.session.model.DeviceHall;
import ru.avilex.session.repository.DeviceHallRepository;
import ru.avilex.session.repository.DeviceRepository;
import ru.avilex.session.validator.DeviceConfigurationValidator;

import java.util.EnumSet;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Service
@RequiredArgsConstructor
public class DeviceService {

    private final DeviceRepository deviceRepository;

    private final DeviceConfigurationValidator deviceConfigurationValidator;

    private final DeviceHallRepository deviceHallRepository;

    private final ResponseMapper responseMapper;

    private final PinCodeService pinCodeService;

    private final NotificationService notificationService;

    private final BigTvDtoBuilderService bigTvDtoBuilderService;

    private final WebSocketPropertiesConfig webSocketProperties;

    public String saveDeviceAndGenerateUuid(DeviceType type, Device device) {
        device.setDeviceType(type.getId());
        device.setStatus(DeviceStatus.CONNECTED.getCode());
        return deviceRepository.save(device).getId().toString();
    }

    public PinCodeAuthResponseDTO authenticate(UUID deviceId, String pin) {
        Device device = deviceRepository.findById(deviceId)
                .orElseThrow(() -> new EntityNotFoundException(String.format(
                        "Device not found by given device_id: %s", deviceId)));
        deviceConfigurationValidator.validate(device);
        pinCodeService.authenticate(device, pin);
        DeviceHall deviceHall = deviceHallRepository.findById(device.getDeviceHall().getUuid())
                .orElseThrow(() -> new EntityNotFoundException(String.format(
                        "Hall not found by given device_hall_id - %s", device.getDeviceHall().getUuid())));

        PinCodeAuthResponseDTO pinCodeAuthResponseDTO = responseMapper.toResponseDTO(deviceHall);
        pinCodeAuthResponseDTO.setSleepTime(device.getDeviceHall().getSleepTime());

        return pinCodeAuthResponseDTO;
    }

    @Transactional
    public void updateDevice(DeviceUpdateRequestDTO requestDTO) {
        Device device = getDevice(requestDTO.getDeviceId());
        DeviceType deviceType = DeviceType.fromCode(device.getDeviceType());
        validateHallId(requestDTO, deviceType);
        validateDisplayHearingStatus(device, deviceType);
        updateDeviceHall(requestDTO, device, deviceType);
        updateDeviceStatus(device);
        updateDeviceParams(requestDTO, device);
        updateStatusAndNotifyBigTv(requestDTO, device, deviceType);
        deviceRepository.save(device);
    }

    @Transactional
    public void updateDeviceList(List<BigTvDeviceUpdateRequestDTO.DeviceUpdateDTO> devices, Integer displayTime) {
        List<UUID> devicesIds = devices.stream().map(BigTvDeviceUpdateRequestDTO.DeviceUpdateDTO::getDeviceId).toList();
        validateIds(devicesIds);
        deviceRepository.updateDisplayTimeForDevicesByIdsAndDeviceType(
                displayTime, devicesIds, DeviceType.BIGTV.getId());
        BigTvUpdatedListMessageDTO bigTvUpdatedListMessageDTO = bigTvDtoBuilderService.createUpdatedDeviceListMessageDto(devicesIds);
        notificationService.notify(webSocketProperties.getBigTvDestination(), bigTvUpdatedListMessageDTO);
    }

    private void validateIds(List<UUID> devicesIds) {
        devicesIds.forEach(id -> {
            if (id == null) {
                throw new IdentificatorValidationException();
            }
            deviceRepository.findById(id).orElseThrow(() -> new EntityNotFoundException(
                    String.format("Device not found by given device_id: %s", id)));
        });
    }

    private void updateStatusAndNotifyBigTv(DeviceUpdateRequestDTO requestDTO, Device device, DeviceType deviceType) {
        if (DeviceType.BIGTV.getId() == deviceType.getId() && device.getDisplayHearingStatus() != null
                && !requestDTO.getDisplayHearingStatus().equals(device.getDisplayHearingStatus())) {
            device.setDisplayHearingStatus(requestDTO.getDisplayHearingStatus());
            BigTvUpdatedConfigMessageDTO bigTvUpdatedConfigMessageDTO = bigTvDtoBuilderService.createUpdatedConfigMessageDto(requestDTO.getDeviceId());
            notificationService.notify(webSocketProperties.getBigTvDestination(), bigTvUpdatedConfigMessageDTO);
        }
    }

    private void validateHallId(DeviceUpdateRequestDTO requestDTO, DeviceType deviceType) {
        if (EnumSet.of(DeviceType.TABLET, DeviceType.SMALLTV).contains(deviceType) && requestDTO.getHallId() == null) {
            throw new BusinessException(HttpStatus.BAD_REQUEST.value(), "hallId is required for devices with type 0 and 1");
        }
    }

    private void validateDisplayHearingStatus(Device device, DeviceType deviceType) {
        if (DeviceType.BIGTV.getId() == deviceType.getId() && device.getDisplayHearingStatus() == null) {
            throw new BusinessException(HttpStatus.BAD_REQUEST.value(),
                    String.format("display hearing status cannot be null for device type %s", DeviceType.BIGTV.getId()));
        }
    }

    private void updateDeviceParams(DeviceUpdateRequestDTO requestDTO, Device device) {
        device.setModel(requestDTO.getModel());
        device.setManufacturer(requestDTO.getManufacturer());
    }

    private void updateDeviceHall(DeviceUpdateRequestDTO requestDTO, Device device, DeviceType deviceType) {
        if (EnumSet.of(DeviceType.TABLET, DeviceType.SMALLTV).contains(deviceType) && requestDTO.getHallId() != null) {
            DeviceHall deviceHall = Optional.of(deviceHallRepository.getReferenceById(requestDTO.getHallId()))
                    .orElseThrow(() -> new EntityNotFoundException(String.format("Device hall not found by given id: %s", requestDTO.getHallId())));
            device.setDeviceHall(deviceHall);
        }
    }

    private void updateDeviceStatus(Device device) {
        if (device.getStatus() == DeviceStatus.CONNECTED.getCode()) {
            device.setStatus(DeviceStatus.ACTIVATED.getCode());
        }
    }

    @Transactional
    public void changeStatus(UUID deviceId, DeviceStatus deviceStatus) {
        Device device = getDevice(deviceId);

        if ((device.getDeviceType() == DeviceType.TABLET.getId() || device.getDeviceType() == DeviceType.SMALLTV.getId()) && deviceStatus == DeviceStatus.DISABLED) {
            device.setDeviceHall(null);
        }
        if (device.getDeviceType() == DeviceType.BIGTV.getId() && deviceStatus == DeviceStatus.DISABLED) {
            device.setDisplayHearingStatus(null);
            device.setDisplayTime(null);
        }
        device.setStatus(deviceStatus.getCode());
        try {
            deviceRepository.save(device);
        } catch (Exception ex) {
            throw new DeviceHallChangeStatusException();
        }
    }

    private Device getDevice(UUID deviceId) {
        return Optional.of(deviceRepository.getReferenceById(deviceId))
                .orElseThrow(() -> new EntityNotFoundException(String.format("Device not found by given id: %s", deviceId)));
    }

}
