package ru.avilex.showcase.repository;

import jakarta.persistence.EntityManager;
import jakarta.persistence.Query;
import jakarta.persistence.Tuple;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import ru.avilex.showcase.dto.*;
import ru.avilex.showcase.dto.response.DeviceHallDTO;
import ru.avilex.showcase.mapper.DeviceHallDTORowMapper;

import java.util.*;
import java.util.stream.Collectors;

@Repository
@RequiredArgsConstructor
public class DeviceHallRepository {

    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final EntityManager entityManager;

    public List<DeviceHallDTO> getListByDeviceTypeId(Integer deviceType) {
        String queryStr = """
                SELECT *
                FROM (SELECT DISTINCT dh.id AS device_hall_id, dh.name
                        FROM session_service.device_hall dh
                        LEFT JOIN session_service.device d ON dh.id = d.device_hall_id
                        WHERE d.device_type NOT IN (COALESCE(:deviceType, 2), 2) OR (d.device_type IS NULL)
                    ) AS dhin
                ORDER BY
                    CASE
                        WHEN dhin.name ~ '^\\D*(\\d+)$' THEN CAST(regexp_replace(dhin.name, '^\\D*(\\d+)$', '\\1') AS INTEGER)
                        ELSE 0
                        END, dhin.name;""";
        MapSqlParameterSource params = new MapSqlParameterSource();
        params.addValue("deviceType", deviceType, java.sql.Types.INTEGER);
        return namedParameterJdbcTemplate.query(queryStr, params, new DeviceHallDTORowMapper());
    }

    private int getOffset(int page, int size) {
        return (page - 1) * size;
    }

    public Optional<PageDTO<HallResponseDTO>> getHalls(Integer deviceType, Integer statusId, Boolean sortDirect, UUID hallId, Integer page, Integer size) {
        try {
            String sql = """
                            SELECT dh.id as hallId,
                                            d.id as id,
                                            d.device_type as deviceType,
                                            d.manufacturer ,
                                            d.model ,
                                            dh.name as hallName,
                                            d.status as statusId,
                                            d.display_hearing_status as displayHearingStatus
                                            FROM session_service.device d 
                                              LEFT JOIN  session_service.device_hall dh ON dh.id = d.device_hall_id
                    """;

            StringBuilder blockWhere = new StringBuilder(" where d.device_type = :deviceType");
            if (Objects.nonNull(statusId)) {
                blockWhere.append(" and d.status = :statusId");
            } else {
                blockWhere.append(" and d.status in (0,1)");
            }
            if (Objects.nonNull(hallId)) {
                blockWhere.append(" and d.device_hall_id = :hallId");
            }

            if (Objects.nonNull(hallId) || Objects.nonNull(statusId)) {
                blockWhere.append(" order by dh.created_on desc");
            } else {
                if (Objects.nonNull(sortDirect)) {
                    if (sortDirect) {
                        blockWhere.append(" order by d.status desc");
                    } else {
                        blockWhere.append(" order by d.status");
                    }
                    blockWhere.append(", dh.created_on desc");
                }
            }

            blockWhere.append(" LIMIT :limit OFFSET :offset");

            sql += blockWhere.toString();

            Query query = entityManager.createNativeQuery(sql, Tuple.class);
            query.setParameter("deviceType", deviceType);
            if (Objects.nonNull(statusId)) {
                query.setParameter("statusId", statusId);
            }
            if (Objects.nonNull(hallId)) {
                query.setParameter("hallId", hallId);
            }

            query.setParameter("limit", size);
            query.setParameter("offset", page == 0 ? page : getOffset(page, size));

            List<Tuple> results = query.getResultList();

            List<HallResponseDTO> result = new ArrayList<>();

            results.forEach(res -> {
                HallResponseDTO hallResponseDTO = new HallResponseDTO(
                        res.get("id", UUID.class),
                        res.get("deviceType", Integer.class),
                        (StringUtils.isNotBlank(res.get("manufacturer", String.class)) ? res.get("manufacturer", String.class) : null),
                        (StringUtils.isNotBlank(res.get("model", String.class)) ? res.get("model", String.class) : null),
                        Objects.nonNull(res.get("hallId", UUID.class)) ? res.get("hallId", UUID.class) : null,
                        (StringUtils.isNotBlank(res.get("hallName", String.class)) ? res.get("hallName", String.class) : null),
                        Integer.valueOf(res.get("statusId", Short.class)),
                        res.get("displayHearingStatus", Short.class)

                );
                result.add(hallResponseDTO);

            });

            String countSql = """
                                
                            SELECT COUNT(*)
                    FROM session_service.device d
                    LEFT JOIN session_service.device_hall dh  ON dh.id = d.device_hall_id
                                    """;

            StringBuilder blockWhereForCount = new StringBuilder(" where d.device_type = :deviceType");
            if (Objects.nonNull(statusId)) {
                blockWhereForCount.append(" and d.status = :statusId");
            } else {
                blockWhereForCount.append(" and d.status in (0,1)");
            }
            if (Objects.nonNull(hallId)) {
                blockWhereForCount.append(" and d.device_hall_id = :hallId");
            }
            countSql += blockWhereForCount.toString();
            Query countQuery = entityManager.createNativeQuery(countSql);
            countQuery.setParameter("deviceType", deviceType);
            if (Objects.nonNull(statusId)) {
                countQuery.setParameter("statusId", statusId);
            }
            if (Objects.nonNull(hallId)) {
                countQuery.setParameter("hallId", hallId);
            }

            int total = ((Number) countQuery.getSingleResult()).intValue();

            PageDTO<HallResponseDTO> pageDTO = new PageDTO<>();
            pageDTO.setContent(result);
            pageDTO.setSize(size);
            pageDTO.setTotalPages(getAllPages(total, size));
            pageDTO.setTotalElements(total);
            pageDTO.setNumber(page == 0 ? page + 1 : page);

            return Optional.of(pageDTO);
        } catch (Exception e) {
            return Optional.empty();
        }
    }

    public Optional<HallCardResponseDTO> getById(UUID id) {
        String sql = """
                        SELECT dh.name AS hallName,
                            dh.sleep_time,
                            dh.pin,
                            dh.notes
                        FROM session_service.device_hall dh
                        WHERE dh.id = :id
                """;

        try {
            var query = entityManager.createNativeQuery(sql, Tuple.class);
            query.setParameter("id", id);
            Tuple result = (Tuple) query.getSingleResult();

            HallCardResponseDTO dto = new HallCardResponseDTO(
                    result.get("hallName", String.class),
                    result.get("sleep_time", Integer.class),
                    result.get("pin", String.class),
                    result.get("notes", String.class)
            );

            var devicesSQL = """
                            select d.id as id,
                            d.device_type as device_type from session_service.device d where d.device_hall_id = :id
                    """;

            var queryDevices = entityManager.createNativeQuery(devicesSQL, Tuple.class);
            queryDevices.setParameter("id", id);
            List<Tuple> devices = (List<Tuple>) queryDevices.getResultList();

            List<DeviceDTO> deviceDTOList = devices.isEmpty()
                    ? Collections.singletonList(new DeviceDTO(UUID.fromString("00000000-0000-0000-0000-000000000000"), 0))
                    : devices.stream()
                    .map(device -> new DeviceDTO(
                            device.get("id", UUID.class),
                            device.get("device_type", Integer.class)
                    ))
                    .collect(Collectors.toList());
            dto.setDeviceDTOList(deviceDTOList);
            return Optional.of(dto);
        } catch (Exception e) {
            return Optional.empty();
        }
    }

    public Optional<PageDTO<HallDeviceDTO>> findAll(int page, int size) {
        String sql = """
                        SELECT dh.id           as device_hall_id
                             , dh.name 
                             , dh.block_reason 
                             , dh.status
                             , dh.notes
                          FROM session_service.device_hall dh
                           WHERE dh.status != 3
                         LIMIT :limit OFFSET :offset
                """;
        String sqlDevices = """
                        select d.id
                             , d.device_type
                          from session_service.device d
                         where d.device_hall_id = :hallId
                """;
        Query query = entityManager.createNativeQuery(sql, Tuple.class);
        query.setParameter("limit", size);
        query.setParameter("offset", page == 0 ? page : getOffset(page, size));

        List<Tuple> results = query.getResultList();
        List<HallDeviceDTO> result = results.stream()
                .map(deviceHall -> {
                    HallDeviceDTO dto = new HallDeviceDTO(
                            deviceHall.get("device_hall_id", UUID.class),
                            deviceHall.get("name", String.class),
                            (StringUtils.isNotBlank(deviceHall.get("block_reason", String.class)) ? deviceHall.get("block_reason", String.class) : ""),
                            Integer.valueOf(deviceHall.get("status", Short.class)),
                            (StringUtils.isNotBlank(deviceHall.get("notes", String.class)) ? deviceHall.get("notes", String.class) : "")
                    );
                    Query queryDevices = entityManager.createNativeQuery(sqlDevices, Tuple.class);
                    queryDevices.setParameter("hallId", dto.getHallId());
                    List<Tuple> devices = queryDevices.getResultList();
                    dto.getDevices().addAll(devices.stream()
                            .map(device -> new DeviceDTO(device.get("id", UUID.class), device.get("device_type", Integer.class)))
                            .toList());
                    return dto;
                }).collect(Collectors.toList());

        Query countQuery = entityManager.createNativeQuery("select count(*) from session_service.device_hall dh  where dh.status != 3");
        int totalElements = ((Number) countQuery.getSingleResult()).intValue();

        PageDTO<HallDeviceDTO> halls = new PageDTO<>();
        halls.setContent(result);
        halls.setNumber(page);
        halls.setSize(size);
        halls.setTotalElements(totalElements);
        halls.setTotalPages(getAllPages(totalElements, size));
        halls.setNumber(page == 0 ? page + 1 : page);

        return Optional.of(halls);
    }

    private int getAllPages(Integer count, Integer size) {
        return count / size + (count % size == 0 ? 0 : 1);
    }

}

