package ru.avilex.showcase.repository;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import ru.avilex.showcase.dto.CompletedSessionDTO;
import ru.avilex.showcase.dto.HallSessionResponseDTO;
import ru.avilex.showcase.dto.HearingSessionDTO;
import ru.avilex.showcase.dto.SessionDTO;
import ru.avilex.showcase.dto.device.SmallTvSessionDTO;
import ru.avilex.showcase.exception.NotFoundException;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Repository
@RequiredArgsConstructor
public class HearingSessionRepository {

    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    public HallSessionResponseDTO findByHallId(String hallId) {
        try {
            String sql = """
                     SELECT h.hall_id,
                            h.hall_name,
                            s.id,
                            h.primary_case_number,
                            s.date_hearing,
                            s.status
                       from hearing_service.hearing h
                      inner join session_service.session s on h.object_id = s.id
                    where h.hall_id = :hall_id and s.status in (0,1)
                    """;

            MapSqlParameterSource params = new MapSqlParameterSource();
            params.addValue("hall_id", hallId);

            List<HearingSessionDTO> sessions = namedParameterJdbcTemplate.query(sql, params, new HearingSessionDTORowMapper());
            if (sessions.isEmpty()) {
                throw new NotFoundException("Could not find session with id " + hallId);
            }

            HearingSessionDTO firstSession = sessions.get(0);
            List<SessionDTO> sessionDTOs = sessions.stream()
                    .map(hearingSession ->
                            new SessionDTO(
                                    hearingSession.getHearingId(),
                                    hearingSession.getPrimaryCaseNumber(),
                                    hearingSession.getHearingStatus(),
                                    hearingSession.getHearingDate()
                            ))
                    .collect(Collectors.toList());

            return HallSessionResponseDTO.builder()
                    .hallId(firstSession.getHallId())
                    .hallName(firstSession.getHallName())
                    .sessions(sessionDTOs)
                    .build();
        } catch (Exception e) {
            throw new NotFoundException("Could not find session with id " + hallId);
        }
    }

    public List<HearingSessionDTO> getSessionsByStatus(Integer status) {
        String sql = """
                 SELECT dh.id as hall_id,
                        dh.name as hall_name,
                        s.hall_id as id,
                        s.primary_case_number,
                        s.date_hearing,
                        s.status
                 from session_service.device_hall dh
                          inner join session_service.session s on dh.id = s.hall_id
                 where s.status = :status
                 order by s.date_hearing
                 limit 14
                """;
        MapSqlParameterSource params = new MapSqlParameterSource();
        params.addValue("status", status);
        return namedParameterJdbcTemplate.query(sql, params, new HearingSessionDTORowMapper());
    }

    public SmallTvSessionDTO getSmallTvActiveSession(UUID deviceId) {
        String sql = """
                SELECT s.primary_case_number
                  from session_service.device d
                 inner join session_service.session s on d.device_hall_id = s.hall_id
                 where d.id = :device_id
                   and s.status = 1
                limit 1
                """;
        MapSqlParameterSource params = new MapSqlParameterSource();
        params.addValue("device_id", deviceId);

        List<String> results = namedParameterJdbcTemplate.query(sql, params, (rs, rowNum) -> rs.getString("primary_case_number"));
        return results.isEmpty() ? null : new SmallTvSessionDTO(results.get(0));
    }

    private static class HearingSessionDTORowMapper implements RowMapper<HearingSessionDTO> {
        @Override
        public HearingSessionDTO mapRow(ResultSet rs, int rowNum) throws SQLException {
            HearingSessionDTO dto = new HearingSessionDTO();
            dto.setHallId(UUID.fromString(rs.getString("hall_id")));
            dto.setHallName(rs.getString("hall_name"));
            dto.setHearingId(UUID.fromString(rs.getString("id")));
            dto.setPrimaryCaseNumber(rs.getString("primary_case_number"));
            dto.setHearingStatus(rs.getInt("status"));
            Timestamp timestamp = rs.getTimestamp("date_hearing");
            if (timestamp != null) {
                dto.setHearingDate(timestamp.toLocalDateTime());
            }
            return dto;
        }
    }

    public List<CompletedSessionDTO> findCompletedSessions(UUID deviceId, Pageable pageable) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("deviceId", deviceId);
        parameters.addValue("limit", pageable.getPageSize());
        parameters.addValue("offset", pageable.getOffset());

        String findCompletedSQL = """

                SELECT s.fact_complete_time AS factCompleteTime, 
                        s.primary_case_number AS primaryCaseNumber,  
                        s.hall_id AS hallId,
                        h.name AS hallName,
                        d.display_hearing_status AS displayHearingStatus,
                        d.status AS status
                         FROM session_service.session s
                        JOIN session_service.device_hall h ON s.hall_id = h.id
                        JOIN session_service.device d ON d.device_hall_id  = s.hall_id
                        WHERE s.status = 2 AND d.id = :deviceId 
                        ORDER BY s.fact_complete_time, h.name 
                        LIMIT :limit OFFSET :offset
                        """;

        List<CompletedSessionDTO> resultList = namedParameterJdbcTemplate.query(
                findCompletedSQL,
                parameters,
                new CompletedSessionRowMapper()
        );
        return resultList;
    }

    private static class CompletedSessionRowMapper implements RowMapper<CompletedSessionDTO> {
        @Override
        public CompletedSessionDTO mapRow(ResultSet rs, int rowNum) throws SQLException {
            return new CompletedSessionDTO(
                    rs.getString("factCompleteTime"),
                    rs.getString("primaryCaseNumber"),
                    UUID.fromString(rs.getString("hallId")),
                    rs.getString("hallName"),
                    rs.getInt("displayHearingStatus"),
                    rs.getShort("status")

            );
        }
    }
}
