import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:al/debug_settings/logic/bloc/environment_settings/environment_settings_cubit.dart';
import 'package:al/framework/environment/server_route.dart';
import 'package:al/framework/extensions/build_context_extension.dart';
import 'package:al/framework/ui/widgets/buttons/al_back_button.dart';
import 'package:al/framework/ui/widgets/al_app_bar.dart';
import 'package:al/framework/ui/widgets/al_radio.dart';

class EnvironmentSettingsPage extends StatefulWidget {
  final Function({required BuildContext context}) onBack;

  const EnvironmentSettingsPage({
    super.key,
    required this.onBack,
  });

  @override
  State<EnvironmentSettingsPage> createState() =>
      _EnvironmentSettingsPageState();
}

class _EnvironmentSettingsPageState extends State<EnvironmentSettingsPage> {
  EnvironmentSettingsCubit get _cubit =>
      context.read<EnvironmentSettingsCubit>();

  final _formKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: ALAppBar(
        titleText: 'Настройки окружения',
        leading: _buildBackButton(context),
      ),
      body: BlocBuilder<EnvironmentSettingsCubit, EnvironmentSettingsState>(
        builder: (context, state) {
          if (state is EnvironmentSettingsInitial) {
            return _buildInitial(context, state);
          } else {
            return Container();
          }
        },
      ),
    );
  }

  Widget _buildBackButton(BuildContext context) {
    return ALBackButton(
      onPressed: () => widget.onBack(context: context),
    );
  }

  Widget _buildInitial(
    BuildContext context,
    EnvironmentSettingsInitial state,
  ) {
    return Form(
      key: _formKey,
      autovalidateMode: AutovalidateMode.always,
      child: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            ..._buildRouteRadios(context, state),
            _buildCustomAddressCheckbox(context, state),
            _buildCustomAddressField(state),
            _buildAnalyticsDebugCheckbox(context, state)
          ],
        ),
      ),
    );
  }

  Widget _buildAnalyticsDebugCheckbox(
    BuildContext context,
    EnvironmentSettingsInitial state,
  ) {
    return SwitchListTile(
      activeColor: context.colors.primary,
      title: const Text('Показывать события аналитики'),
      value: state.isAnalyticsDebugEnabled ?? false,
      onChanged: (value) => _analyticsDebugEnabledChange(isEnabled: value),
    );
  }

  void _analyticsDebugEnabledChange({required bool isEnabled}) {
    _cubit.setAnalyticsDebugEnabled(isEnabled);
  }

  List<Widget> _buildRouteRadios(
    BuildContext context,
    EnvironmentSettingsInitial state,
  ) {
    return (state.routes ?? []).map((route) {
      return GestureDetector(
        onTap: () => _cubit.selectRoute(route),
        child: ListTile(
          title: Text(_getRouteName(route)),
          leading: ALRadio<ServerRoute?>(
            value: route,
            groupValue: state.selectedRoute,
            onChanged: (ServerRoute? value) => _cubit.selectRoute(value),
          ),
        ),
      );
    }).toList();
  }

  String _getRouteName(ServerRoute route) {
    switch (route) {
      case ServerRoute.dev:
        return "Development";
      case ServerRoute.prod:
        return "Production";
    }
  }

  Widget _buildCustomAddressCheckbox(
    BuildContext context,
    EnvironmentSettingsInitial state,
  ) {
    return SwitchListTile(
      activeColor: context.colors.primary,
      title: const Text("Использовать нестандартный адрес"),
      value: state.isCustomServerAddressEnabled ?? false,
      onChanged: (value) => _cubit.setCustomAddressEnabled(value),
    );
  }

  Widget _buildCustomAddressField(EnvironmentSettingsInitial state) {
    return Padding(
      padding: const EdgeInsets.only(left: 17, right: 17),
      child: TextFormField(
        initialValue: state.customServerAddress,
        validator: _validateUrl,
        onFieldSubmitted: (value) {
          if (_validateUrl(value) == null) {
            _cubit.setCustomAddress(value);
          }
        },
      ),
    );
  }

  String? _validateUrl(value) {
    try {
      Uri.parse(value);
      return null;
    } on Exception {
      return "Неправильный адрес";
    }
  }
}
