import 'package:al/framework/environment/build_type.dart';
import 'package:al/framework/environment/server_route.dart';
import 'package:al/framework/res/al_contstants.dart';
import 'package:al/framework/storages/preferences_storage.dart';

class Environment {
  final IPreferencesStorage _preferencesStorage;
  final BuildType _buildType;

  Environment({
    required BuildType buildType,
    required IPreferencesStorage preferencesStorage,
  })  : _buildType = buildType,
        _preferencesStorage = preferencesStorage;

  /// Доступно ли отладочное меню в настройках
  bool get isDeveloperToolsAvailable {
    switch (_buildType) {
      case BuildType.internal:
        return true;
      case BuildType.store:
      default:
        return false;
    }
  }

  /// Серверный контур, выбранный при запуске приложения
  ServerRoute get serverRoute {
    if (isDeveloperToolsAvailable) {
      return _preferencesStorage.serverRoute ?? defaultServerRoute;
    } else {
      return defaultServerRoute;
    }
  }

  /// Серверный контур, используемый по-умолчанию
  ServerRoute get defaultServerRoute {
    switch (_buildType) {
      case BuildType.internal:
        return ServerRoute.dev;
      case BuildType.store:
      default:
        return ServerRoute.prod;
    }
  }

  /// Изменение выбранного серверного контура
  Future<void> setServerRoute(ServerRoute? route) async {
    _preferencesStorage.serverRoute = route;
    await _preferencesStorage.save();
  }

  /// Адрес сервера для отправки запросов
  String get serverAddress {
    final customAddress = _preferencesStorage.customServerAddress;
    if (isCustomServerAddressEnabled && customAddress != null) {
      return customAddress;
    }

    switch (serverRoute) {
      case ServerRoute.dev:
        return ALConstants.serverAddressDev;
      case ServerRoute.prod:
        return ALConstants.serverAddressProd;
    }
  }

  /// Включен ли нестандартный адрес сервера
  bool get isCustomServerAddressEnabled {
    return isDeveloperToolsAvailable &&
        _preferencesStorage.customServerAddress != null &&
        _preferencesStorage.isCustomServerEnabled == true;
  }

  String? get customServerAddress {
    return _preferencesStorage.customServerAddress;
  }

  /// Установка нестандартного адреса сервера
  Future<void> setCustomServerAddress(String? address) async {
    _preferencesStorage.customServerAddress = address;
    await _preferencesStorage.save();
  }

  /// Включение использования нестандартного адреса сервера
  Future<void> setCustomServerAddressEnabled(bool enabled) async {
    _preferencesStorage.isCustomServerEnabled = enabled;
    await _preferencesStorage.save();
  }

  /// Доступна ли отладка отправки событий аналитики
  bool get isAnalyticsDebugEnabled {
    return _preferencesStorage.isAnalyticsDebugEnabled == true &&
        isDeveloperToolsAvailable;
  }

  /// Включение отладки отправки событий аналитики
  Future<void> setAnalyticsDebugEnabled({required bool isEnabled}) async {
    _preferencesStorage.isAnalyticsDebugEnabled = isEnabled;
    await _preferencesStorage.save();
  }
}
