import 'package:al/framework/network/i_paging_response.dart';
import 'package:al/framework/network/page_metadata.dart';

typedef ChangeOnPage = void Function(int);
typedef CurrentPageLoading<T> = Future<IPagingResponse<T>> Function(
  RequestPaging<T> paging,
);
typedef PageLoadingCompletion = void Function();
typedef PageLoadingError = void Function(Exception);
typedef PageReloadCondition<T> = bool Function(RequestPaging<T>);
typedef PageLoadingFilter<T> = bool Function(T);
typedef ReloadCompletion = void Function();

class RequestPaging<T> {
  static const _defaultOnPage = 10;

  /// Здесь должна происходить загрузка данных.
  ///
  /// Количество загружаемых элементов в странице должно совпадать со значением pageInfo.onPage
  final CurrentPageLoading<T> currentPageLoading;

  /// Это замыкание вызывается после окончания загрузки страниц.
  ///
  /// Если фильтр pageLoadingFilter отсек часть элементов и страница не последняя - загружается
  /// следующая страница. В этом слу˚чае pageLoadingCompletion вызовется после того как будет
  /// набрано минимально необходимое число элементов для новой порции.
  final PageLoadingCompletion? pageLoadingCompletion;

  /// Это замыкание вызывается в случае ошибки загрузки очередной страницы.
  final PageLoadingError? pageLoadingError;

  /// Это замыкание вызывается для каждого нового элемента после окончания загрузки страницы.
  ///
  /// Если оно возвращает false, то элемент не попадает ни в список загруженных элементов items
  /// ни в список новых элементов newItems. При этом, если полученное количество новых элементов
  /// меньше требуемой длины страницы pageInfo.onPage, происходит загрузка следующей страницы.
  final PageLoadingFilter<T>? pageLoadingFilter;

  /// Это замыкание вызывается после окончания загрузки или перезагрузки
  final ReloadCompletion? reloadCompletion;

  /// Количество элементов, запрашиваемых с сервера при загрузке очередной страницы.
  final int onPage;

  final List<T> items = [];

  List<T> _newItems = [];

  var _pageNumber = 1;
  var _hasNextPage = true;

  RequestPaging({
    this.onPage = _defaultOnPage,
    required this.currentPageLoading,
    this.pageLoadingCompletion,
    this.pageLoadingError,
    this.pageLoadingFilter,
    this.reloadCompletion,
  });

  List<T> get newItems => _newItems;

  int get pageNumber => _pageNumber;

  bool get hasNextPage => _hasNextPage;

  /// Загружает первую страницу пагинации.
  ///
  /// Если пагинация уже содержала какие-либо элементы, предыдущее состояние
  /// оказывается перезаписано.
  Future<List<T>> reload() async {
    _hasNextPage = true;
    _pageNumber = 1;
    await _loadPage(overrideItems: true);
    reloadCompletion?.call();

    return items;
  }

  /// Пытается перезагрузить пагинацию, получив с сервера то же число страниц,
  /// что было раньше.
  Future<List<T>> reloadPages() async {
    final targetPageNumber = pageNumber;

    _pageNumber = 1;
    _hasNextPage = true;

    await _loadPagesRecursivelyUntil(
      (paging) => paging.pageNumber == targetPageNumber,
    );
    reloadCompletion?.call();

    return items;
  }

  /// Загружает пагинацию, пока не будет выполнено некоторое условие.
  Future<List<T>> reloadUntil(PageReloadCondition<T> condition) async {
    _pageNumber = 1;
    _hasNextPage = true;

    await _loadPagesRecursivelyUntil(condition);
    reloadCompletion?.call();

    return items;
  }

  /// Загружает пагинацию, пока страницы не закончатся
  Future<List<T>> reloadAll() => reloadUntil((_) => false);

  /// Загружает следующую страницу.
  ///
  /// Метод возвращает ВСЕ элементы, а не последнюю порцию.
  Future<List<T>> loadNextPage() async {
    await _loadPage(overrideItems: false);

    return items;
  }

  /// Вызов этой функции выставит признак окончания загрузки.
  /// Можно использовать внутри currentPageLoading, чтобы принудительно
  /// завершить загрузку, даже если пришла полная страница данных и в ином случае
  /// произошла бы попытка загрузки следующей страницы.
  void notifyLastPage() {
    _hasNextPage = false;
  }

  void clear() {
    items.clear();
    _newItems.clear();
    _pageNumber = 1;
    _hasNextPage = true;
  }

  Future<void> _loadPage({
    int? originalPage,
    List<T> filteredItems = const [],
    required bool overrideItems,
  }) async {
    final originalPageNumber = originalPage ?? pageNumber;
    try {
      final newPage = await currentPageLoading(this);
      // hasNextPage мог стать false во время загрузки
      _hasNextPage = hasNextPage && newPage.meta.hasNextPage;
      if (hasNextPage) {
        _pageNumber++;
      }
      final newItems = newPage.items;
      final newFilteredItems = pageLoadingFilter != null
          ? newItems.where(pageLoadingFilter!).toList()
          : newItems;
      final allFilteredItems = [...filteredItems, ...newFilteredItems];

      // Если фильтр отсек несколько элементов - экран может быть не заполнен полностью
      // и пагинация сломается. Так что загружаем следующие страницы, пока не наберется
      // нужное для заполнения экрана число элементов.
      if (allFilteredItems.length >= onPage || !hasNextPage) {
        if (overrideItems) {
          items.clear();
        }
        items.addAll(allFilteredItems);

        _newItems = allFilteredItems;

        pageLoadingCompletion?.call();
      } else {
        await _loadPage(
          originalPage: originalPageNumber,
          filteredItems: allFilteredItems,
          overrideItems: overrideItems,
        );
      }
    } on Exception catch (e) {
      _pageNumber = originalPageNumber;
      pageLoadingError?.call(e);
      rethrow;
    }
  }

  /// Метод будет загружать страницы до тех пор пока условие until не будет выполнено или
  /// до тех пор пока не будут получены все страницы.
  Future<void> _loadPagesRecursivelyUntil(
    PageReloadCondition<T> condition,
  ) async {
    await _loadPage(overrideItems: pageNumber == 1);

    if (hasNextPage && condition(this) == false) {
      await _loadPagesRecursivelyUntil(condition);
    }
  }
}

extension _NextPageCheck on PageMetadata {
  bool get hasNextPage => currentPage < pageCount;
}
