import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:al/framework/extensions/build_context_extension.dart';
import 'package:al/framework/res/images/al_asset_change.dart';
import 'package:al/framework/types/al_device_type.dart';
import 'package:flutter_svg/flutter_svg.dart';

/// Графический ассет, зашитый в приложение.
///
/// Владея этим объектом, можно получить виджет с изображением, вызвав метод
/// [buildWidget].
class ALImageAsset extends Equatable {
  // Должен быть true если приложение запущено не из основного модуля, и требует
  // доступ к ассетам.
  static var enableModuleImport = false;

  /// Флаг того, что этот ассет векторный.
  final bool isVector;

  /// Имя файла ассета.
  final String name;

  /// Желаемая плотность пикселей варианта ассета, используемая при выводе на
  /// экран.
  ///
  /// Актуально только для растровой графики, и в большинстве случаев приложение
  /// должно определять плотность пикселей самостоятельно, в зависимости от
  /// экрана. Если нужно именно такое поведение, этот параметр должен иметь
  /// значение `null`.
  final int? pixelRatio;

  /// Размер, который само изображение попытается занять на экране. Не включает
  /// в себя [padding].
  final Size? size;

  /// Собственные отступы вокруг изображения при выводе на экран.
  final EdgeInsets? padding;

  /// Изменения, которые должны быть применены к ассету при выводе на экран
  /// телефона в ландшафтной ориентации.
  final ALAssetChange? landscape;

  /// Изменения, которые должны быть применены к ассету при выводе на экран
  /// планшета с шириной меньше 1024 логических пикселей.
  final ALAssetChange? tablet;

  /// Изменения, которые должны быть применены к ассету при выводе на экран
  /// планшета с шириной 1024 логических пикселей или больше.
  final ALAssetChange? wideTablet;

  /// Изменения, которые должны быть применены к ассету при выводе на экран,
  /// если применена светлая тема оформления.
  ///
  /// Изменения по яркости применяются после изменений по размеру.
  final ALAssetChange? light;

  /// Изменения, которые должны быть применены к ассету при выводе на экран,
  /// если применена темная тема оформления.
  ///
  /// Изменения по яркости применяются после изменений по размеру.
  final ALAssetChange? dark;

  /// Векторный ассет.
  const ALImageAsset.vector(
    this.name, {
    this.size,
    this.padding,
    this.landscape,
    this.tablet,
    this.wideTablet,
    this.light,
    this.dark,
  })  : isVector = true,
        pixelRatio = null;

  /// Растровый ассет.
  const ALImageAsset.raster(
    this.name, {
    this.size,
    this.padding,
    this.pixelRatio,
    this.landscape,
    this.tablet,
    this.wideTablet,
    this.light,
    this.dark,
  }) : isVector = false;

  /// Возвращает виджет, соответствующий графическому ассету.
  ///
  /// [fit] - способ вписывания иллюстрации в выделенное на экране пространство.
  /// По умолчанию, `BoxFit.contain`.
  ///
  /// [padding] - пустое пространство вокруг иллюстрации. Подготовленные для
  /// ландшафтной ориентации ассеты содержат заранее заданное пустое место
  /// вокруг картинки, которое будет использовано, если этот параметр не задан
  /// или равен `null`.
  ///
  /// [deviceType] - тип устройства, для которого предназначен ассет (телефон,
  /// телефон в ландшафтной ориентации, планшет, планшет с широким экраном).
  ///
  /// [change] - изменение, которое нужно внести в ассет перед выводом на
  /// экран. Например, если нужно убрать собственный `padding` векторного ассета,
  /// это поле должно иметь значение `ALAssetChange.padding(EdgeInsets.zero)`.
  /// Это изменение применяется после стандартной адаптации ассета к типу
  /// устройства и яркости темы.
  Widget buildWidget(
    BuildContext context, {
    BoxFit fit = BoxFit.contain,
    Color? color,
    ALAssetChange? change,
  }) {
    final sizeChange = context.deviceType.pickValue(
      phone: null,
      landscape: landscape,
      tablet: tablet,
      wideTablet: wideTablet,
    );
    final brightnessChange = context.brightness.pickValue(
      light: light,
      dark: dark,
    );

    final deviceChange = ALAssetChange.combine(sizeChange, brightnessChange);
    final appliedChange = ALAssetChange.combine(deviceChange, change);

    return _ALAssetFormatter(this, appliedChange)
        .buildImage(color: color, fit: fit);
  }

  Future<void> precache() async {
    if (isVector) {
      final loader = SvgAssetLoader(
        _ALAssetFormatter(this).getAssetName(),
        packageName: enableModuleImport ? 'al' : null,
      );
      await svg.cache.putIfAbsent(
        loader.cacheKey(null),
        () => loader.loadBytes(null),
      );
    } else {
      // Растровые картинки загружаются с аппаратным ускорением, и как правило
      // никакого мерцания не вызывают.
    }
  }

  ALImageAsset copyWith({
    String? name,
    int? pixelRatio,
    Size? size,
    EdgeInsets? padding,
    ALAssetChange? landscape,
    ALAssetChange? tablet,
    ALAssetChange? wideTablet,
    ALAssetChange? light,
    ALAssetChange? dark,
  }) {
    return isVector
        ? ALImageAsset.vector(
            name ?? this.name,
            size: size ?? this.size,
            landscape: landscape ?? this.landscape,
            tablet: tablet ?? this.tablet,
            wideTablet: wideTablet ?? this.wideTablet,
            light: light ?? this.light,
            dark: dark ?? this.dark,
          )
        : ALImageAsset.raster(
            name ?? this.name,
            size: size ?? this.size,
            pixelRatio: pixelRatio ?? this.pixelRatio,
            padding: padding ?? this.padding,
            landscape: landscape ?? this.landscape,
            tablet: tablet ?? this.tablet,
            wideTablet: wideTablet ?? this.wideTablet,
            light: light ?? this.light,
            dark: dark ?? this.dark,
          );
  }

  @override
  List<Object?> get props => [
    isVector,
    name,
    landscape,
    tablet,
    wideTablet,
  ];
}

class _ALAssetFormatter {
  final ALImageAsset asset;
  final ALAssetChange? change;

  _ALAssetFormatter(this.asset, [this.change]);

  Widget buildImage({BoxFit? fit, Color? color}) {
    if (asset.isVector) {
      return _buildVectorImage(fit, color);
    } else {
      return _buildRasterImage(fit, color);
    }
  }

  Widget _buildVectorImage(BoxFit? fit, Color? color) {
    final imageName = getAssetName();
    final imageSize = scaledSize;

    final imageWidth = imageSize.width.isFinite ? imageSize.width : null;
    final imageHeight = imageSize.height.isFinite ? imageSize.height : null;

    final colorFilter =
        color != null ? ColorFilter.mode(color, BlendMode.srcIn) : null;

    return Padding(
      padding: scaledPadding,
      child: SvgPicture.asset(
        imageName,
        package: ALImageAsset.enableModuleImport ? 'al' : null,
        colorFilter: colorFilter,
        width: imageWidth,
        height: imageHeight,
        placeholderBuilder: (_) {
          return SizedBox(
            width: imageWidth,
            height: imageHeight,
          );
        },
        fit: fit ?? BoxFit.contain,
      ),
    );
  }

  Widget _buildRasterImage(BoxFit? fit, Color? color) {
    // Масштаб изображения определяется тем, какие масштабы нами предусмотрены.
    // Сейчас мы кладем в пакет ресурсы масштабов 1x, 2x и 3x.
    final assetScale = asset.pixelRatio ??
        WidgetsBinding.instance.platformDispatcher.views.single.devicePixelRatio
            .ceilToDouble()
            .clamp(1, 3)
            .toInt();

    final imageName = getAssetName(scale: assetScale);
    final imageSize = scaledSize;

    // Стандартный загрузчик изображений AssetImage не обеспечивает
    // производительности, при которой загрузка картинок происходит незаметно
    // для пользователя. Поэтому мы используем намного более быстрый
    // ExactAssetImage.

    final package = ALImageAsset.enableModuleImport ? 'al' : null;

    return Padding(
      padding: scaledPadding,
      child: Image(
        image: ExactAssetImage(
          imageName,
          scale: assetScale.toDouble(),
          package: package,
        ),
        fit: fit,
        width: imageSize.width.isFinite ? imageSize.width : null,
        height: imageSize.height.isFinite ? imageSize.height : null,
        color: color,
      ),
    );
  }

  String getAssetName({int scale = 1}) {
    final name = asset.name;
    final variant = change?.variant ?? '';

    if (name.isEmpty) {
      return '';
    }

    final pathComponents = name.split('/');
    final directories = pathComponents.take(pathComponents.length - 1);
    final fileName = '${pathComponents.last}.${asset.isVector ? 'svg' : 'png'}';

    final path = [
      'assets',
      asset.isVector ? 'svg' : 'images',
      ...directories,
      if (variant.isNotEmpty) variant,

      // Растры масштабов, отличающихся от 1.0, хранятся в директориях вида
      // '%.1fx', находящихся по тому же пути, что и ресурс в базовом масштабе (1x).
      // Т.е. для ресурса 'assets/images/app_icon.png' версию в масштабе 3x
      // следует загружать по пути 'assets/images/3.0x/app_icon.png'.
      if (!asset.isVector && scale > 1) '${scale.toStringAsFixed(1)}x',
      fileName,
    ];

    return path.join('/');
  }

  double get scale => change?.scale ?? 1;

  Size get scaledSize => (change?.size ?? asset.size ?? Size.infinite) * scale;

  EdgeInsets get scaledPadding =>
      (change?.padding ?? asset.padding ?? EdgeInsets.zero) * scale;
}

extension _Utility on Brightness {
  T? pickValue<T>({T? light, T? dark}) {
    switch (this) {
      case Brightness.dark:
        return dark;
      case Brightness.light:
        return light;
    }
  }
}
