import 'package:al/framework/environment/server_route.dart';
import 'package:al/framework/formatters/al_date_formatter.dart';
import 'package:shared_preferences/shared_preferences.dart';

abstract class IPreferencesStorage {
  /// Признак того, что приложение было запущено после последней переустановки.
  bool? appWasLaunched;

  /// Выбранный контур сервера.
  ServerRoute? serverRoute;

  /// Указанный вручную адрес сервера. Используется для отладочного прокси-сервера.
  String? customServerAddress;

  /// Нужно ли использовать указанный вручную адрес сервера.
  bool? isCustomServerEnabled;

  /// Отладка отправки событий аналитики
  bool? isAnalyticsDebugEnabled;

  Future<void> load();
  Future<void> save();
  Future<void> clear({bool fully = false});
}

class PreferencesStorage extends IPreferencesStorage {
  // Признак, было ли хранилище загружено после инициализации
  var _isLoaded = false;

  final _appWasLaunchedKey = "applicationWasLaunchedOnce";
  final _serverRouteKey = "serverRoute";
  final _customServerAddressKey = "customServerAddress";
  final _isCustomServerEnabledKey = "isCustomServerEnabled";
  final _isAnalyticsDebugEnabledKey = "isAnalyticsDebugEnabled";

  @override
  Future<void> load() async {
    final storage = await SharedPreferences.getInstance();
    await storage.reload();

    appWasLaunched = storage.getBool(_appWasLaunchedKey) ?? false;

    serverRoute = ServerRouteSerializer.fromString(
        storage.getString(_serverRouteKey) ?? '');

    customServerAddress = storage.getString(_customServerAddressKey);
    isCustomServerEnabled = storage.getBool(_isCustomServerEnabledKey) ?? false;
    isAnalyticsDebugEnabled =
        storage.getBool(_isAnalyticsDebugEnabledKey) ?? false;

    _isLoaded = true;
  }

  @override
  Future<void> save() async {
    if (!_isLoaded) {
      throw ('''
      Обнаружена попытка сохранить PreferencesStorage без предварительной загрузки.
      Это может привести к тому что часть данных в хранилище будет потеряна.
      ''');
    }

    final storage = await SharedPreferences.getInstance();

    await Future.wait([
      storage.setOptionalBool(_appWasLaunchedKey, appWasLaunched),
      storage.setOptionalString(
          _serverRouteKey, ServerRouteSerializer.routeToString(serverRoute)),
      storage.setOptionalString(_customServerAddressKey, customServerAddress),
      storage.setOptionalBool(_isCustomServerEnabledKey, isCustomServerEnabled),
      storage.setOptionalBool(
          _isAnalyticsDebugEnabledKey, isAnalyticsDebugEnabled),
    ]);
  }

  @override
  Future<void> clear({bool fully = false}) async {
    final storage = await SharedPreferences.getInstance();

    if (fully) {
      storage.clear();
      return;
    }

    // Сбрасываем ключи по одному, т.к. часть настроек нужно сохранять
    await Future.wait([]);
  }
}

extension _NullConvenience on SharedPreferences {
  // ignore: unused_element
  Future<bool> setOptionalBool(String key, bool? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setBool(key, value);
    }
  }

  // ignore: unused_element
  Future<bool> setOptionalInt(String key, int? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setInt(key, value);
    }
  }

  // ignore: unused_element
  Future<bool> setOptionalString(String key, String? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setString(key, value);
    }
  }

  // ignore: unused_element
  Future<bool> setOptionalDouble(String key, double? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setDouble(key, value);
    }
  }

  // ignore: unused_element
  Future<bool> setOptionalStringList(String key, List<String>? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setStringList(key, value);
    }
  }

  // ignore: unused_element
  Future<bool> setOptionalDate(String key, DateTime? value) {
    if (value == null) {
      return remove(key);
    } else {
      return setString(key, ALDateFormatter.format(value, ALDateFormat.dmyhms));
    }
  }
}
