import 'package:flutter_secure_storage/flutter_secure_storage.dart';

abstract class ISecureStorage {
  /// Сессия пользователя.
  String? uuid;

  Future<void> clear({bool fully = false});
  Future<void> load();
  Future<void> save();
}

class SecureStorage extends ISecureStorage {
  final _iosOptions = const IOSOptions(
    accessibility: KeychainAccessibility.first_unlock_this_device,
    accountName: null,
  );

  final _uuidKey = "uuid";

  // Признак, было ли хранилище загружено после инициализации
  var _isLoaded = false;

  @override
  Future<void> clear({bool fully = false}) async {
    uuid = null;

    const storage = FlutterSecureStorage();

    if (fully) {
      storage.deleteAll(
        aOptions: const AndroidOptions(resetOnError: true),
      );
      return;
    }

    await _delete(storage, key: _uuidKey);
  }

  @override
  Future<void> load() async {
    const storage = FlutterSecureStorage();
    uuid = await _read(storage, key: _uuidKey);

    _isLoaded = true;
  }

  @override
  Future<void> save() async {
    if (!_isLoaded) {
      throw ("""
      Обнаружена попытка сохранить SecureStorage без предварительной загрузки.
      Это может привести к тому что часть данных в хранилище будет потеряна.
      """);
    }
    const storage = FlutterSecureStorage();
    await _write(storage, key: _uuidKey, value: uuid);
  }

  Future<String?> _read(
    FlutterSecureStorage storage, {
    required String key,
  }) async {
    return await storage.read(key: key, iOptions: _iosOptions);
  }

  Future<void> _write(
    FlutterSecureStorage storage, {
    required String key,
    required String? value,
  }) async {
    if (value == null) {
      await storage.delete(key: key, iOptions: _iosOptions);
    } else {
      await storage.write(key: key, value: value, iOptions: _iosOptions);
    }
  }

  Future<void> _delete(
    FlutterSecureStorage storage, {
    required String key,
  }) async {
    return await storage.delete(key: key, iOptions: _iosOptions);
  }
}
