import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:al/framework/extensions/build_context_extension.dart';

enum ALAlertPlatformStyle { ios, android }

class ALAlert extends StatelessWidget {
  final String text;
  final List<ALAlertButton> buttons;
  final ALAlertPlatformStyle? platformStyle;

  const ALAlert({
    super.key,
    required this.text,
    required this.buttons,
    this.platformStyle,
  });

  @override
  Widget build(BuildContext context) {
    final style = platformStyle ??
        (Platform.isIOS
            ? ALAlertPlatformStyle.ios
            : ALAlertPlatformStyle.android);
    if (style == ALAlertPlatformStyle.ios) {
      return _CupertinoDialog(text: text, buttons: buttons);
    } else {
      return _MaterialDialog(text: text, buttons: buttons);
    }
  }

  static Future<void> show(
    String text, {
    required BuildContext context,
    required List<ALAlertButton> buttons,
    ALAlertPlatformStyle? platformStyle,
  }) {
    return showDialog<void>(
      context: context,
      barrierDismissible: false,
      builder: (context) => ALAlert(
        text: text,
        buttons: buttons,
        platformStyle: platformStyle,
      ),
    );
  }
}

class ALAlertButton {
  final String text;
  final Color? color;
  final VoidCallback? action;

  ALAlertButton({
    required this.text,
    this.color,
    this.action,
  });
}

class _CupertinoDialog extends StatelessWidget {
  final String text;
  final List<ALAlertButton> buttons;

  const _CupertinoDialog({
    required this.text,
    required this.buttons,
  });

  @override
  Widget build(BuildContext context) {
    return CupertinoAlertDialog(
      content: Text(text),
      actions: buttons
          .map(
            (button) => CupertinoDialogAction(
              child: Text(
                button.text,
                style: TextStyle(color: button.color),
              ),
              onPressed: () {
                Navigator.of(context).pop();
                if (button.action != null) {
                  // Дождаемся окончания анимации скрытия
                  Future.delayed(kThemeAnimationDuration, button.action);
                }
              },
            ),
          )
          .toList(),
    );
  }
}

class _MaterialDialog extends StatelessWidget {
  final String text;
  final List<ALAlertButton> buttons;

  const _MaterialDialog({
    required this.text,
    required this.buttons,
  });

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      content: Text(text),
      actions: buttons
          .map(
            (button) => _buildButton(
              context: context,
              button: button,
            ),
          )
          .toList(),
    );
  }

  Widget _buildButton({required BuildContext context, required ALAlertButton button}) {
    return TextButton(
      onPressed: () {
        Navigator.of(context).pop();
        button.action?.call();
      },
      style: TextButton.styleFrom(
        foregroundColor: button.color ?? context.colors.primary,
      ),
      child: Text(button.text),
    );
  }
}
