import 'package:flutter/material.dart';
import 'package:al/framework/extensions/build_context_extension.dart';
import 'package:al/framework/res/images/al_asset_change.dart';
import 'package:al/framework/res/images/al_image_asset.dart';
import 'package:al/framework/ui/widgets/al_badged_icon.dart';
import 'package:al/framework/ui/widgets/al_bottom_panel.dart';

class ALBottomTabBarItem {
  final String title;
  final ALImageAsset icon;
  final String? badgeText;

  const ALBottomTabBarItem({
    required this.title,
    required this.icon,
    this.badgeText,
  });

  ALBottomTabBarItem copyWithBadgeText(String? text) {
    return ALBottomTabBarItem(title: title, icon: icon, badgeText: text);
  }
}

class ALBottomTabBar extends StatelessWidget {
  static const height = 56.0;
  static const iconSize = 24.0;
  static const maxTabWidth = 75.0;

  final List<ALBottomTabBarItem> tabs;
  final int selectedIndex;

  /// Цвет невыбранной иконки.
  ///
  /// Если не задан, будет применен `slateGrey`.
  final Color? defaultIconColor;

  /// Цвет выбранной иконки.
  ///
  /// Если не задан, будет применен `dodgerBlue`.
  final Color? selectedIconColor;

  /// Цвет невыбранного текста.
  ///
  /// Если не задан, будет применен `slateGrey`.
  final Color? defaultTextColor;

  /// Цвет выбранного текста.
  ///
  /// Если не задан, будет применен `dodgerBlueLink`.
  final Color? selectedTextColor;

  /// Цвет фона панели.
  ///
  /// Если не задан, будет применен `whiteCard`.
  final Color? backgroundColor;

  /// Цвет значка с уведомлением.
  ///
  /// Если не задан, будет применен `orange`.
  final Color? badgeColor;

  /// Цвет текста на значке с уведомлением.
  ///
  /// Если не задан, будет применен `whiteAccent`.
  final Color? badgeTextColor;

  final Function(int) onSelectTab;

  const ALBottomTabBar({
    super.key,
    required this.tabs,
    required this.selectedIndex,
    this.defaultIconColor,
    this.selectedIconColor,
    this.defaultTextColor,
    this.selectedTextColor,
    this.backgroundColor,
    this.badgeColor,
    this.badgeTextColor,
    required this.onSelectTab,
  });

  @override
  Widget build(BuildContext context) {
    return ClipRect(
      clipper: _DirectionalShadowClipper(),
      child: ALBottomPanel.bottomTabBar(
        context: context,
        height: height,
        child: _buildNavigationBar(context),
      ),
    );
  }

  BottomNavigationBar _buildNavigationBar(BuildContext context) {
    return BottomNavigationBar(
      type: BottomNavigationBarType.fixed,
      currentIndex: selectedIndex,
      selectedItemColor: selectedTextColor ?? context.colors.dodgerBlueLink,
      selectedLabelStyle: context.textStyles.caption2Regular,
      unselectedItemColor: defaultTextColor ?? context.colors.slateGrey,
      unselectedLabelStyle: context.textStyles.caption2Regular,
      backgroundColor: backgroundColor ?? context.colors.whiteCard,
      onTap: (int index) => onSelectTab(index),
      elevation: 0,
      items: _buildTabs(context),
    );
  }

  List<BottomNavigationBarItem> _buildTabs(BuildContext context) {
    return tabs.map((tab) {
      return BottomNavigationBarItem(
        label: tab.title,
        icon: _buildBadgedIcon(tab, context, isSelected: false),
        activeIcon: _buildBadgedIcon(tab, context, isSelected: true),
      );
    }).toList();
  }

  Widget _buildBadgedIcon(
    ALBottomTabBarItem tab,
    BuildContext context, {
    required bool isSelected,
  }) {
    return ALBadgedIcon(
      badgeText: tab.badgeText,
      badgeColor: badgeColor ?? context.colors.orange,
      badgeTextColor: badgeTextColor ?? context.colors.fixed.whiteBackground,
      iconSize: iconSize,
      maxTabWidth: maxTabWidth,
      icon: tab.icon.buildWidget(
        context,
        color: _getIconColor(context, isSelected: isSelected),
        change: ALAssetChange.square(iconSize),
      ),
    );
  }

  Color _getIconColor(BuildContext context, {required bool isSelected}) {
    if (isSelected) {
      return selectedIconColor ?? context.colors.dodgerBlue;
    } else {
      return defaultIconColor ?? context.colors.slateGrey;
    }
  }
}

// Маска, при помощи которой таббар обрезает ненужные тени с боков и внизу
// панели.
class _DirectionalShadowClipper extends CustomClipper<Rect> {
  // Важно, чтобы это число не находилось в окрестности предельных значений
  // типа double. Это связано с тем, что под капотом Flutter иногда оперирует
  // координатами углов прямоугольников, а иногда шириной/высотой, и в такие
  // моменты есть риск выйти за рамки.
  static const arbitrarilyLargeNegative = -43897634.0;

  @override
  Rect getClip(Size size) {
    return Rect.fromLTRB(0, arbitrarilyLargeNegative, size.width, size.height);
  }

  @override
  bool shouldReclip(covariant CustomClipper<Rect> oldClipper) {
    return oldClipper is! _DirectionalShadowClipper;
  }
}
