import 'package:flutter/material.dart';
import 'package:al/framework/extensions/build_context_extension.dart';
import 'package:keyboard_actions/keyboard_actions.dart';

/// В этот виджет нужно обернуть поля ввода, между которыми нужно выполнять переход
/// вперед-назад.
/// 1. Нужно создать FocusNode для каждого поля ввода и передать их в конструкторах полей ввода.
/// 2. Нужно создать AgKeyboardActions и передать в него список FocusNode в том же порядке,
/// в котором поля ввода на экране следуют друг за другом.
/// 3. В поле child AgKeyboardActions передается контейнер с полями ввода, например список.
///

class AGKeyboardActions extends StatefulWidget {
  final List<FocusNode> focusNodes;
  final Widget child;

  const AGKeyboardActions({
    super.key,
    required this.focusNodes,
    required this.child,
  });

  /// Вариант виджета, который накладывает на [child] ограничения по высоте.
  ///
  /// Базовая версия передает [child]'у неограниченную высоту, что удобно
  /// в большинстве случаев; однако иногда возникает необходимость выравнять
  /// по нижнему краю экрана какой-нибудь элемент (к примеру, нижнюю панель),
  /// и в таком случае практичнее использовать этот вариант.
  AGKeyboardActions.constrained({
    super.key,
    required this.focusNodes,
    required Widget child,
  }) : child = _buildConstrainedContainer(child);

  @override
  State<AGKeyboardActions> createState() => _AGKeyboardActionsState();

  static Widget _buildConstrainedContainer(Widget child) {
    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        height: constraints.minHeight,
        child: child,
      );
    });
  }
}

class _AGKeyboardActionsState extends State<AGKeyboardActions> {
  @override
  Widget build(BuildContext context) {
    return KeyboardActions(
      config: _getKeyboardActionsConfig(),
      child: widget.child,
    );
  }

  KeyboardActionsConfig _getKeyboardActionsConfig() {
    return KeyboardActionsConfig(
      keyboardActionsPlatform: KeyboardActionsPlatform.IOS,
      keyboardBarColor:
          context.isDarkMode ? context.colors.white : Colors.grey[200],
      nextFocus: true,
      actions: widget.focusNodes.map((node) => _getCloseAction(node)).toList(),
    );
  }

  KeyboardActionsItem _getCloseAction(FocusNode focusNode) {
    return KeyboardActionsItem(
      focusNode: focusNode,
      toolbarButtons: [
        (node) {
          return GestureDetector(
            onTap: () => node.unfocus(),
            child: Container(
              color: Colors.transparent,
              padding: const EdgeInsets.only(right: 16, bottom: 1),
              child: Text(
                "Готово",
                style: context.textStyles.textBold,
              ),
            ),
          );
        },
      ],
    );
  }
}
