import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:al/launcher/logic/bloc/launcher_cubit.dart';

class LauncherPage extends StatefulWidget {
  final Widget Function() onGetActivationPage;
  final Widget Function() onGetAuthPage;
  final Widget Function() onGetLoggedInPage;

  const LauncherPage({
    super.key,
    required this.onGetActivationPage,
    required this.onGetAuthPage,
    required this.onGetLoggedInPage,
  });

  @override
  State<StatefulWidget> createState() => _LauncherPageState();
}

class _LauncherPageState extends State<LauncherPage> {
  final _activationRequiredKey = GlobalKey<NavigatorState>();
  final _authRequiredKey = GlobalKey<NavigatorState>();
  final _loggedInKey = GlobalKey<NavigatorState>();

  GlobalKey<NavigatorState>? _currentStateKey;

  LauncherCubit get _cubit => context.read<LauncherCubit>();

  @override
  void initState() {
    super.initState();
    _cubit.notifyAppStarted();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: _buildBackButtonHandler(
        context: context,
        child: BlocBuilder<LauncherCubit, LauncherState>(
          builder: (context, state) {
            switch (state) {
              case LauncherInitial():
                return Container(color: Colors.blue);
              case LauncherActivationInitial():
                return _buildActivationPage();
              case LauncherAuthenticationInitial():
                return _buildAuthenticationPage();
              case LauncherLoggedInInitial():
                return _buildLoggedInPage();
            }
          },
        ),
      ),
    );
  }

  Widget _buildBackButtonHandler({
    required BuildContext context,
    required Widget child,
  }) {
    // На iOS кнопки назад нет, возвращаем сам виджет
    if (Platform.isIOS) {
      return child;
    }
    return WillPopScope(
      onWillPop: () async {
        // Если вернулся true - возврат произошел или был запрещен WillPopScope внутри виджета.
        // Если false - это возврат из корневого виджета, надо закрывать приложение.
        final popResult =
            await _currentStateKey?.currentState?.maybePop() ?? false;
        // Если вернуть true - приложение закроется
        return !popResult;
      },
      child: child,
    );
  }

  Widget _buildAuthenticationPage() {
    return _buildNavigatorStack(
      _authRequiredKey,
      (context) => widget.onGetAuthPage(),
    );
  }

  Widget _buildActivationPage() {
    return _buildNavigatorStack(
      _activationRequiredKey,
      (context) => widget.onGetActivationPage(),
    );
  }

  Widget _buildLoggedInPage() {
    return _buildNavigatorStack(
      _loggedInKey,
      (context) => widget.onGetLoggedInPage(),
    );
  }

  Widget _buildNavigatorStack(
    GlobalKey<NavigatorState> key,
    WidgetBuilder widgetBuilder,
  ) {
    _currentStateKey = key;
    return Navigator(
      key: key,
      onGenerateRoute: (settings) {
        return MaterialPageRoute(builder: widgetBuilder, settings: settings);
      },
    );
  }
}
