import 'package:al/appbar/res/appbar_styles.dart';
import 'package:al/framework/formatters/al_date_formatter.dart';
import 'package:al/session_list/entities/hall_session.dart';
import 'package:al/session_list/logic/bloc/session_list_cubit.dart';
import 'package:al/session_list/res/session_list_styles.dart';
import 'package:al/session_list/types/hall_session_status.dart';
import 'package:al/session_list/ui/session_list/session_action_dialog.dart';
import 'package:al/session_list/ui/session_list/session_button.dart';
import 'package:al/session_list/ui/session_list/session_logout_dialog.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class SessionListPage extends StatefulWidget {
  final PreferredSizeWidget Function({
    required BuildContext context,
    Widget? title,
    List<Widget>? actions,
  }) onGetAppBar;

  const SessionListPage({
    super.key,
    required this.onGetAppBar,
  });

  @override
  State<SessionListPage> createState() => _SessionListPageState();
}

class _SessionListPageState extends State<SessionListPage> {
  SessionListCubit get _cubit => context.read<SessionListCubit>();

  @override
  void initState() {
    _cubit.initialize();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SessionListCubit, SessionListState>(
      builder: _buildCurrentState,
    );
  }

  Widget _buildCurrentState(BuildContext context, SessionListState state) {
    return Scaffold(
      appBar: widget.onGetAppBar(
        context: context,
        title: _buildAppBarTitle(state),
        actions: [
          _buildLogoutButton(hallName: state.hallName),
        ],
      ),
      body: _buildContent(state),
    );
  }

  Widget _buildContent(SessionListState state) {
    if (state is SessionListInProgress) {
      return const Center(
        child: CircularProgressIndicator(),
      );
    } else {
      return Padding(
        padding: const EdgeInsets.symmetric(horizontal: 30),
        child: SingleChildScrollView(
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 20),
            child: Table(
              columnWidths: const <int, TableColumnWidth>{
                0: FlexColumnWidth(0.8),
                1: FlexColumnWidth(0.4),
                2: FlexColumnWidth(0.4),
                3: FlexColumnWidth(0.4),
              },
              defaultVerticalAlignment: TableCellVerticalAlignment.middle,
              children: [
                _buildHeaderTableRow(),
                ..._buildSessions(state.hallSessions),
              ],
            ),
          ),
        ),
      );
    }
  }

  TableRow _buildHeaderTableRow() {
    return TableRow(
      children: [
        const SizedBox(),
        TableCell(
          verticalAlignment: TableCellVerticalAlignment.middle,
          child: Align(
            alignment: Alignment.center,
            child: Text(
              "ВРЕМЯ",
              style: SessionListStyles.columnTitle,
            ),
          ),
        ),
        TableCell(
          child: Align(
            alignment: Alignment.center,
            child: Text(
              "СТАТУС",
              style: SessionListStyles.columnTitle,
            ),
          ),
        ),
        TableCell(
          child: Align(
            alignment: Alignment.center,
            child: Text(
              "НОМЕР ДЕЛА",
              style: SessionListStyles.columnTitle,
            ),
          ),
        ),
      ],
    );
  }

  List<TableRow> _buildSessions(List<HallSession> sessions) {
    final mappedSessions = sessions.map(_mapSession);

    return mappedSessions.toList(growable: false);
  }

  TableRow _mapSession(HallSession session) {
    return TableRow(
      children: [
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 8.0),
          child: SessionButton(
            hallSession: session,
            onPressed: () => _onButtonPressed(session),
          ),
        ),
        _buildTableRowText(
          text: ALDateFormatter.format(
            session.hearingDate,
            ALDateFormat.hhmm,
          ),
        ),
        _buildTableRowText(
          text: session.status.getString(),
        ),
        _buildTableRowText(
          text: session.primaryCaseNumber,
        ),
      ],
    );
  }

  Widget _buildTableRowText({required String text}) {
    return Align(
      alignment: Alignment.center,
      child: Text(
        text,
        style: SessionListStyles.columnText,
        textAlign: TextAlign.center,
      ),
    );
  }

  Widget _buildAppBarTitle(SessionListState state) {
    return Text(
      "Зал: ${state.hallName}",
      style: AppBarStyles.appBarText,
    );
  }

  void _onButtonPressed(HallSession session) {
    showDialog(
      context: context,
      builder: (innerContext) => SessionActionDialog(
        hallSession: session,
        cubit: _cubit,
      ),
    );
  }

  Widget _buildLogoutButton({
    required String hallName,
  }) {
    return Padding(
      padding: const EdgeInsets.only(right: 24.0),
      child: IconButton(
        onPressed: () => showDialog(
          context: context,
          builder: (innerContext) => SessionLogoutDialog(
            hallName: hallName,
            cubit: _cubit,
          ),
        ),
        icon: const Icon(
          Icons.exit_to_app,
          color: Colors.white,
        ),
        iconSize: 36.0,
        highlightColor: Colors.white.withOpacity(0.1),
      ),
    );
  }
}
