import 'dart:async';

import 'package:flutter/material.dart';
import 'package:al/framework/errors/error_handler/error_handler_formatter.dart';
import 'package:al/framework/errors/exceptions/connection_failed_exception.dart';
import 'package:al/framework/errors/exceptions/message_exception.dart';
import 'package:al/framework/errors/exceptions/no_auth_exception.dart';
import 'package:al/framework/errors/exceptions/no_connection_exception.dart';
import 'package:al/framework/errors/exceptions/server_message_exception/server_message_exception.dart';
import 'package:al/framework/framework_dependencies.dart';
import 'package:al/framework/log/crash_logger.dart';
import 'package:al/framework/log/log.dart';
import 'package:al/framework/errors/error_handler/error_handler.dart';
import 'package:al/framework/ui/widgets/buttons/al_button.dart';

class ErrorHandlerPage extends StatefulWidget {
  const ErrorHandlerPage({super.key});

  @override
  State<ErrorHandlerPage> createState() => _ErrorHandlerPageState();
}

class _ErrorHandlerPageState extends State<ErrorHandlerPage> {
  late final ErrorHandler _errorHandler;
  late final List<Widget> _buttons;

  @override
  void initState() {
    Log.configure(isLogEnabled: false, crashLogger: CrashLoggerStub());
    _errorHandler = ErrorHandler(
      formatter: ErrorHandlerFormatter(),
      logoutCaller: const _FrameworkErrorHandlerLogoutCaller(),
    );
    _buttons = _buildButtons();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: ListView.separated(
        itemBuilder: (context, index) => _buttons[index],
        separatorBuilder: (_, __) => const SizedBox(height: 10),
        itemCount: _buttons.length,
      ),
    );
  }

  List<Widget> _buildButtons() {
    return [
      _buildButton(
        text: "Нет подключения",
        onPressed: () => _showWidget(
          child: _errorHandler.buildErrorWidget(
            exception: const NoConnectionException(),
            onRetry: () => Navigator.of(context).pop(),
          ),
        ),
      ),
      _buildButton(
        text: "Технические работы",
        onPressed: () => _showWidget(
          child: _errorHandler.buildErrorWidget(
            exception: ServerMessageException(
              rawMessage: "",
              message: null,
              code: 10000,
            ),
            onRetry: () => Navigator.of(context).pop(),
          ),
        ),
      ),
      _buildButton(
        text: "Ошибка входа",
        onPressed: () => _showWidget(
          child: _errorHandler.buildErrorWidget(
            exception: const NoAuthException(),
            onRetry: () {},
          ),
        ),
      ),
      _buildButton(
        text: "Сообщение на ConnectionFailedException",
        onPressed: () => _errorHandler.showException(
          ConnectionFailedException(Exception("Какая-то ошибка")),
          context: context,
        ),
      ),
      _buildButton(
        text: "Сообщение на MessageException",
        onPressed: () => _errorHandler.showException(
          const MessageException("Какая-то ошибка"),
          context: context,
        ),
      ),
      _buildButton(
        text: "Сообщение на NoAuthException",
        onPressed: () => _errorHandler.showException(
          const NoAuthException(),
          context: context,
        ),
      ),
      _buildButton(
        text: "Сообщение на NoConnectionException",
        onPressed: () => _errorHandler.showException(
          const NoConnectionException(),
          context: context,
        ),
      ),
      _buildButton(
        text: "Сообщение на ServerMessageException",
        onPressed: () => _errorHandler.showException(
          ServerMessageException(
              rawMessage: "Какая-то ошибка", code: 10000, message: null),
          context: context,
        ),
      ),
      _buildButton(
        text: "Сообщение на прочие ошибки",
        onPressed: () => _errorHandler.showException(
          Exception("Какая-то ошибка"),
          context: context,
        ),
      ),
    ];
  }

  Widget _buildButton({required String text, required VoidCallback onPressed}) {
    return Container(
      padding: const EdgeInsets.fromLTRB(10, 0, 10, 10),
      width: double.infinity,
      child: ALButton(text: text, onPressed: onPressed),
    );
  }

  void _showWidget({required Widget child}) {
    Navigator.of(context).push(
      MaterialPageRoute(builder: (context) {
        return Scaffold(
          body: child,
        );
      }),
    );
  }
}

class _FrameworkErrorHandlerLogoutCaller implements IFrameworkErrorHandlerLogoutCaller {
  const _FrameworkErrorHandlerLogoutCaller();

  @override
  FutureOr<void> errorHandlerLogOut() {}
}
