import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:altv/debug_settings/logic/bloc/feature_settings/feature_settings_cubit.dart';
import 'package:altv/debug_settings/logic/entities/feature_settings_item.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/res/images/al_images.dart';
import 'package:altv/framework/ui/widgets/buttons/al_back_button.dart';
import 'package:altv/framework/ui/widgets/al_app_bar.dart';
import 'package:altv/framework/ui/widgets/al_switch.dart';

class FeatureSettingsPage extends StatefulWidget {
  final Function({required BuildContext context}) onBack;

  const FeatureSettingsPage({
    Key? key,
    required this.onBack,
  }) : super(key: key);

  @override
  State<FeatureSettingsPage> createState() => _FeatureSettingsPageState();
}

class _FeatureSettingsPageState extends State<FeatureSettingsPage> {
  FeatureSettingsCubit get _cubit => context.read<FeatureSettingsCubit>();

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<FeatureSettingsCubit, FeatureSettingsState>(
      builder: (context, state) {
        return Scaffold(
          appBar: ALAppBar(
            titleText: "Настройки фич",
            leading: _buildBackButton(context),
            actions: _buildAppBarActions(state),
          ),
          body: SafeArea(child: _buildContents(context, state)),
        );
      },
    );
  }

  Widget _buildBackButton(BuildContext context) {
    return ALBackButton(onPressed: () {
      widget.onBack.call(context: context);
    });
  }

  List<Widget> _buildAppBarActions(FeatureSettingsState state) {
    if (state.hasOverriddenSettings) {
      return [
        IconButton(
          icon: ALImages.iconRecentlyUsed.buildWidget(
            context,
            color: context.colors.dark,
          ),
          onPressed: () => _cubit.resetOverrides(),
        )
      ];
    } else {
      return [];
    }
  }

  Widget _buildContents(BuildContext context, FeatureSettingsState state) {
    return ListView.builder(
      itemCount: state.items.length,
      itemBuilder: (context, index) =>
          _buildFeatureItem(context, state.items[index]),
    );
  }

  Widget _buildFeatureItem(BuildContext context, FeatureSettingsItem item) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        Padding(
          padding: const EdgeInsets.symmetric(horizontal: 20.0, vertical: 10.0),
          child: Row(
            children: [
              Expanded(child: _buildFeatureDescription(item)),
              if (item.overrideValue != null) _buildFeatureResetButton(item),
              _buildFeatureSwitch(item),
            ],
          ),
        ),
        Container(
          margin: const EdgeInsets.only(left: 20.0),
          height: 1.0,
          color: context.colors.coolGrey,
        ),
      ],
    );
  }

  Column _buildFeatureDescription(FeatureSettingsItem item) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        Text(
          item.description,
          style: context.textStyles.bodySmallRegular,
        ),
        if (item.overrideValue != null)
          Text('(по умолчанию: ${item.isReleased ? 'вкл.' : 'выкл.'})',
              style: context.textStyles.labelLargeLight),
      ],
    );
  }

  ALSwitch _buildFeatureSwitch(FeatureSettingsItem item) {
    return ALSwitch(
      value: item.isAvailable,
      onChanged: (state) => _cubit.setFeatureOverride(item.id, value: state),
    );
  }

  GestureDetector _buildFeatureResetButton(FeatureSettingsItem item) {
    return GestureDetector(
        onTap: () => _cubit.setFeatureOverride(item.id, value: null),
        child: Padding(
          padding: const EdgeInsets.only(right: 10.0),
          child: ALImages.iconRecentlyUsed
              .buildWidget(context, color: context.colors.dark),
        ));
  }
}
