import 'package:altv/framework/environment/build_type.dart';
import 'package:altv/framework/environment/server_route.dart';
import 'package:altv/framework/res/al_contstants.dart';
import 'package:altv/framework/storages/preferences_storage.dart';

class Environment {
  final IPreferencesStorage preferencesStorage;
  final BuildType buildType;

  Environment({
    required this.buildType,
    required this.preferencesStorage,
  });

  /// Доступно ли отладочное меню в настройках
  bool get isDeveloperToolsAvailable {
    switch (buildType) {
      case BuildType.internal:
        return true;
      case BuildType.store:
      default:
        return false;
    }
  }

  /// Серверный контур, выбранный при запуске приложения
  ServerRoute get serverRoute {
    if (isDeveloperToolsAvailable) {
      return preferencesStorage.serverRoute ?? defaultServerRoute;
    } else {
      return defaultServerRoute;
    }
  }

  /// Серверный контур, используемый по-умолчанию
  ServerRoute get defaultServerRoute {
    switch (buildType) {
      case BuildType.internal:
        return ServerRoute.development;
      case BuildType.store:
      default:
        return ServerRoute.production;
    }
  }

  /// Изменение выбранного серверного контура
  Future<void> setServerRoute(ServerRoute? route) async {
    preferencesStorage.serverRoute = route;
    await preferencesStorage.save();
  }

  /// Адрес сервера для отправки запросов
  String get serverAddress {
    final customAddress = preferencesStorage.customServerAddress;
    if (isCustomServerAddressEnabled && customAddress != null) {
      return customAddress;
    }

    switch (serverRoute) {
      case ServerRoute.development:
        return  ALConstants.serverAddressDev;
      case ServerRoute.production:
        return  ALConstants.serverAddressProd;
    }
  }

  /// Включен ли нестандартный адрес сервера
  bool get isCustomServerAddressEnabled {
    return isDeveloperToolsAvailable &&
        preferencesStorage.customServerAddress != null &&
        preferencesStorage.isCustomServerEnabled == true;
  }

  String? get customServerAddress {
    return preferencesStorage.customServerAddress;
  }

  /// Установка нестандартного адреса сервера
  Future<void> setCustomServerAddress(String? address) async {
    preferencesStorage.customServerAddress = address;
    await preferencesStorage.save();
  }

  /// Включение использования нестандартного адреса сервера
  Future<void> setCustomServerAddressEnabled(bool enabled) async {
    preferencesStorage.isCustomServerEnabled = enabled;
    await preferencesStorage.save();
  }

  /// Доступна ли отладка отправки событий аналитики
  bool get isAnalyticsDebugEnabled {
    return preferencesStorage.isAnalyticsDebugEnabled == true &&
        isDeveloperToolsAvailable;
  }

  /// Включение отладки отправки событий аналитики
  Future<void> setAnalyticsDebugEnabled({required bool isEnabled}) async {
    preferencesStorage.isAnalyticsDebugEnabled = isEnabled;
    await preferencesStorage.save();
  }
}
