
import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:altv/framework/log/crash_logger.dart';
import 'package:logger/logger.dart';

class Log {
  // Логер, который просто печатает текст.
  // Полезен для логирования сетевых запросов.
  static Logger? _simpleLogger;

  // Логер, который обводит каждую строчку в рамочку и показывает эмодзи
  static Logger? _prettyLogger;

  // Логер для отправки ошибок в Crashlytics, AppMetric и т.д.
  static ICrashLogger? _crashLogger;

  static void configure({
    required bool isLogEnabled,
    required ICrashLogger crashLogger,
  }) {
    _simpleLogger = Logger(
      printer: _LogPrinter(
        printEmojis: false,
        noBoxingByDefault: true,
      ),
      filter: _LogFilter(isLogEnabled: isLogEnabled),
    );

    _prettyLogger = Logger(
      printer: _LogPrinter(
        printEmojis: true,
        noBoxingByDefault: false,
      ),
      filter: _LogFilter(isLogEnabled: isLogEnabled),
    );

    _crashLogger = crashLogger;
  }

  /// Информация
  static void i(String message) {
    _prettyLogger?.i(message);
  }

  /// Ошибки
  static void e(String message) {
    _prettyLogger?.e(message);
  }

  /// Предупреждения
  static void w(String message) {
    _prettyLogger?.w(message);
  }

  /// Отладка
  static void d(String message) {
    _prettyLogger?.d(message);
  }

  /// Подробная информация
  static void v(String message) {
    _prettyLogger?.t(message);
  }

  /// Без форматирования
  static void simple(String message) {
    _simpleLogger?.t(message);
  }

  /// Вывод JSON. Если [pretty], то в рамочке
  static void json(String json, {var pretty = false}) {
    const encoder = JsonEncoder.withIndent("  ");
    final jsonBody = jsonDecode(json);
    final message = encoder.convert(jsonDecode(jsonEncode(jsonBody)));
    if (pretty) {
      _prettyLogger?.t(message);
    } else {
      _simpleLogger?.t(message);
    }
  }

  /// Вывод FormData. Если [pretty], то в рамочке
  static void formData(FormData formData, {var pretty = false}) {
    final message = StringBuffer();
    message.write(
      formData.fields.map((field) => "${field.key}:${field.value}").join(",\n"),
    );
    for (final file in formData.files) {
      final multipartFile = file.value;
      message.write("${file.key}: {\n");
      message.write("  filename:${multipartFile.filename},\n");
      message.write("  length:${multipartFile.length},\n");
      message.write("  contentType:${multipartFile.contentType},\n");
      message.write("}");
    }
    if (pretty) {
      _prettyLogger?.t(message);
    } else {
      _simpleLogger?.t(message);
    }
  }

  /// Exception с отправкой в Crashlytics, AppMetric и т.д.
  static Future<void> crashException(
    Exception e, {
    required StackTrace stack,
    String? message,
  }) async {
    if (_crashLogger != null) {
      await _crashLogger!.logException(e, stack: stack, message: message);
    } else {
      simple(e.toString());
      if (message != null) {
        simple("Message: $message");
      }
      simple("Stack trace:\n$stack");
    }
  }

  /// Error с отправкой в Crashlytics, AppMetric и т.д.
  static Future<void> crashError(
    Error e, {
    required StackTrace stack,
    String? message,
  }) async {
    if (_crashLogger != null) {
      await _crashLogger!.logError(e, stack: stack, message: message);
    } else {
      simple(e.toString());
      if (message != null) {
        simple("Message: $message");
      }
      simple("Stack trace:\n$stack");
    }
  }
}

class _LogFilter extends LogFilter {
  final bool isLogEnabled;

  _LogFilter({required this.isLogEnabled});

  @override
  bool shouldLog(Object event) => isLogEnabled;
}

class _LogPrinter extends PrettyPrinter {
  _LogPrinter({
    required bool printEmojis,
    required bool noBoxingByDefault,
  }) : super(
    // глубина отображаемой вложенности вызовов
    methodCount: 1,
    // глубина отображаемой вложенности вызовов при отображении ошибки
    errorMethodCount: 8,
    lineLength: 100,
    colors: false,
    printEmojis: printEmojis,
    printTime: false,
    noBoxingByDefault: noBoxingByDefault,
  );

  @override
  String? formatStackTrace(StackTrace? stackTrace, int? methodCount) {
    final incrementedCount = (methodCount ?? 0) + 1;
    final lines = stackTrace.toString().split("\n");
    final formatted = <String>[];
    var count = 0;
    for (final line in lines) {
      if (_discardDeviceStackTraceLine(line) ||
          _discardWebStackTraceLine(line)) {
        continue;
      }

      if (count != 0) {
        formatted.add(
            "#${count - 1}   ${line.replaceFirst(RegExp(r"#\d+\s+"), '')}");
      }
      if (++count == incrementedCount) {
        break;
      }

      if (formatted.isEmpty) {
        return null;
      }
    }

    return formatted.join("\n");
  }

  bool _discardDeviceStackTraceLine(String line) {
    final match = _deviceStackTraceRegex.matchAsPrefix(line);
    if (match == null) {
      return false;
    }
    return match.group(2)?.startsWith("package:logger") ?? false;
  }

  bool _discardWebStackTraceLine(String line) {
    final match = _webStackTraceRegex.matchAsPrefix(line);
    if (match == null) {
      return false;
    }
    return (match.group(1)?.startsWith("packages/logger") ?? false) ||
        (match.group(1)?.startsWith("dart-sdk/lib") ?? false);
  }

  static final _deviceStackTraceRegex = RegExp(r"#[0-9]+\s+(.+) \((\S+)\)");

  static final _webStackTraceRegex = RegExp(r"^((packages|dart-sdk)/\S+/)");
}
