import 'dart:io';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:dio/dio.dart';
import 'package:altv/framework/environment/environment.dart';
import 'package:altv/framework/errors/exceptions/connection_failed_exception.dart';
import 'package:altv/framework/errors/exceptions/message_exception.dart';
import 'package:altv/framework/errors/exceptions/no_connection_exception.dart';
import 'package:altv/framework/errors/exceptions/server_message_exception/server_message_exception.dart';
import 'package:altv/framework/errors/exceptions/server_message_exception/server_message_exception_response.dart';
import 'package:altv/framework/errors/exceptions/server_page_not_found_exception.dart';
import 'package:altv/framework/formatters/al_date_formatter.dart';
import 'package:altv/framework/log/log.dart';
import 'package:altv/framework/network/i_request.dart';
import 'package:altv/framework/network/i_response.dart';
import 'package:altv/framework/res/al_error_strings.dart';
import 'package:typed_json/typed_json.dart';

export 'i_request.dart';
export 'i_response.dart';

/// Уровень логирования сетевых запросов.
/// Нужен, чтобы не логировать запросы загрузки файлов на сервер
enum RequestLoggingLevel { full, header, none }

/// Уровень логирования сетевых ответов.
/// Нужен, чтобы не логировать длинные ответы с телом картинок и видео
enum ResponseLoggingLevel { full, header, none }

class RequestSender {
  static const defaultReceiveTimeout = Duration(seconds: 60);
  static const defaultSendTimeout = Duration(seconds: 60);

  final String _openSeparator = 'v' * 100;
  final String _closeSeparator = '^' * 100;

  final Environment environment;

  RequestSender({
    required this.environment,
  });

  Future<T> send<T extends IBaseResponse>(
      IRequest request,
      T response, {
        RequestLoggingLevel requestLogging = RequestLoggingLevel.full,
        ResponseLoggingLevel responseLogging = ResponseLoggingLevel.full,
        bool updateTokensOnAuthError = true,
        Duration receiveTimeout = defaultReceiveTimeout,
        Duration sendTimeout = defaultSendTimeout,
      }) async {
    await _checkConnection();

    final address = _getAddress(request);
    final options = _getOptions(
      request: request,
      responseType: response.responseType,
      receiveTimeout: receiveTimeout,
      sendTimeout: sendTimeout,
    );
    final body = await request.getRawBody();

    _logRequest(
      requestLogging: requestLogging,
      httpMethod: request.httpMethod,
      address: address,
      options: options,
      body: body,
    );

    final httpResponse = await _sendRequest(
      httpMethod: request.httpMethod,
      address: address,
      options: options,
      body: body,
    );

    _logResponse(
      responseLogging: responseLogging,
      httpMethod: request.httpMethod,
      address: address,
      httpResponse: httpResponse,
    );

    try {
      // Теоретически, проверка ошибок тоже может бросить FormatException
      _checkResponseErrors(httpResponse);

      await response.rawResult(httpResponse.data);
    } on FormatException catch (e) {
      Log.crashException(e, stack: StackTrace.current);
      if (environment.isDeveloperToolsAvailable) {
        throw (MessageException(e.toString()));
      } else {
        throw (const MessageException(
          'Сервер временно недоступен. Попробуйте позже.',
        ));
      }
    } on Error catch (e) {
      Log.crashError(e, stack: StackTrace.current);
      if (environment.isDeveloperToolsAvailable) {
        throw (MessageException(e.toString()));
      } else {
        throw (const MessageException(ALErrorStrings.defaultExceptionText));
      }
    }

    return response;
  }

  Future<void> _checkConnection() async {
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult.contains(ConnectivityResult.none)) {
      throw const NoConnectionException();
    }
  }

  String _getAddress(IRequest request) {
    final baseUrl = request.baseUrlOverride ?? environment.serverAddress;
    final method = request.apiMethod;
    final params = _getRequestUriParameters(request);
    final query = Uri(queryParameters: params).query;

    final questionAndQuery = '${query.isNotEmpty ? '?' : ''}$query';
    final address = '$method$questionAndQuery';
    final fullAddress = '$baseUrl${address.isNotEmpty ? '/' : ''}$address';

    return fullAddress;
  }

  Map<String, dynamic> _getRequestUriParameters(IRequest request) {
    return request.getParameters().map(
          (key, value) => MapEntry(
        key,
        value is String
            ? value
            : value is Iterable<String>
            ? value
            : value.toString(),
      ),
    );
  }

  Options _getOptions({
    required IRequest request,
    required ResponseType responseType,
    required Duration receiveTimeout,
    required Duration sendTimeout,
  }) {
    final headers = <String, String>{};

    final requestHeaders = request.getHeaders();
    headers.addAll(requestHeaders);

    return Options(
      receiveTimeout: receiveTimeout,
      sendTimeout: sendTimeout,
      contentType: requestHeaders.containsKey(Headers.contentTypeHeader)
          ? requestHeaders[Headers.contentTypeHeader]
          : Headers.jsonContentType,
      responseType: responseType,
      headers: headers,
    );
  }

  void _logRequest({
    required RequestLoggingLevel requestLogging,
    required HttpMethod httpMethod,
    required String address,
    required Options options,
    required Object body,
  }) {
    if (requestLogging != RequestLoggingLevel.none) {
      Log.simple(_openSeparator);
      Log.simple(
        '${_getCurrentTimeText()} ${_getHttpMethodLogName(httpMethod)}: $address',
      );
      Log.simple('HEADERS: ${options.headers.toString()}');

      if (requestLogging == RequestLoggingLevel.full) {
        if (body is Json && body.isExist) {
          Log.json(body.asPrettyString());
        } else if (body is FormData) {
          Log.formData(body);
        } else if (body is String) {
          Log.simple(body);
        }
      }

      Log.simple(_closeSeparator);
    }
  }

  void _logResponse({
    required ResponseLoggingLevel responseLogging,
    required HttpMethod httpMethod,
    required String address,
    required Response httpResponse,
  }) {
    if (responseLogging != ResponseLoggingLevel.none) {
      Log.simple(_openSeparator);
      Log.simple(
        'RESPONSE ${_getCurrentTimeText()} ${_getHttpMethodLogName(httpMethod)}: $address',
      );
      Log.simple(
        '${httpResponse.statusCode} ${httpResponse.statusMessage}',
      );
      if (responseLogging == ResponseLoggingLevel.full) {
        try {
          Log.json(Json(httpResponse.data).asPrettyString());
        } on FormatException {
          Log.simple(httpResponse.data.toString());
        }
      }

      Log.simple(_closeSeparator);
    }
  }

  String _getHttpMethodLogName(HttpMethod httpMethod) {
    switch (httpMethod) {
      case HttpMethod.post:
        return 'POST';
      case HttpMethod.get:
        return 'GET';
      case HttpMethod.delete:
        return 'DELETE';
    }
  }

  String _getCurrentTimeText() {
    return ALDateFormatter.format(DateTime.now(), ALDateFormat.ymdhms);
  }

  Future<Response> _sendRequest({
    required HttpMethod httpMethod,
    required String address,
    required Options options,
    required Object body,
  }) async {
    try {
      final bodyContent = body is Json ? body.asString() : body;
      final dio = Dio();

      switch (httpMethod) {
        case HttpMethod.post:
          return await dio.post(address, data: bodyContent, options: options);
        case HttpMethod.get:
          return await dio.get(address, options: options);
        case HttpMethod.delete:
          return await dio.delete(address, options: options);
      }
    } on DioException catch (e) {
      Log.crashException(e, stack: StackTrace.current);

      switch (e.type) {
        case DioExceptionType.badResponse:
        // На этом этапе мы не производим обработку ошибок, связанных с ненормальным
        // (!=200) кодом ответа, и внутри этого метода считаем такие ответы вариантом
        // корректной работы.
          return e.response!;

        case DioExceptionType.connectionTimeout:
        case DioExceptionType.sendTimeout:
        case DioExceptionType.receiveTimeout:
        // время ожидания может быть превышено в тч и при использовании vpn (connectTimeout, receiveTimeout)
          throw (const MessageException(
            'Превышено время ожидания ответа. Проверьте подключение к интернету или попробуйте позже.',
          ));

        case DioExceptionType.cancel:
        //  возможно, этот тип ошибки возникнет при использовнаии vpn
        //  поэтому, может быть лучше для продуктовой среды возвращать defaultExceptionText
          throw (MessageException(e.message ?? ''));

        case DioExceptionType.badCertificate:
        case DioExceptionType.connectionError:
        case DioExceptionType.unknown:
        // e.error is SocketException возвращается в случае, если нет интернета или используется vpn
        // e.error is HandshakeException возвращается в случае, если используется vpn
        // возможно, могут быть и другие e.error при использовании vpn
          throw (const NoConnectionException());
      }
    } on SocketException catch (e) {
      Log.crashException(e, stack: StackTrace.current);
      throw (ConnectionFailedException(e));
    } on FormatException catch (e) {
      Log.crashException(e, stack: StackTrace.current);
      if (environment.isDeveloperToolsAvailable) {
        throw (MessageException(e.toString()));
      } else {
        throw (const MessageException(
          'Сервер временно недоступен. Попробуйте позже.',
        ));
      }
    }
  }

  void _checkResponseErrors(Response httpResponse) {
    final statusCode = httpResponse.statusCode!;
    // Если 404, то пробрасываем Not Found
    if (statusCode == 404) {
      throw ServerPageNotFoundException(code: statusCode);
    }

    // Если 400, то должен прийти объект с описанием ошибки
    if (statusCode == 400) {
      final json = Json(httpResponse.data);
      final commonError = ServerMessageExceptionResponse.fromJson(json);
      throw ServerMessageException(
        code: commonError?.status ?? statusCode,
        message: commonError?.message,
        rawMessage: commonError?.rawMessage,
      );
    }

    // Если код не 200, то считаем что произошла ошибка.
    if (statusCode < 200 || statusCode >= 300) {
      throw ServerMessageException(
        code: statusCode,
        rawMessage: '',
        message: null,
      );
    }
  }
}