import 'package:flutter/material.dart';

/// Тип устройства, на котором запущено приложение.
/// Набор устройств будет зависеть от макетов конкретного приложения
enum ALDeviceType {
  phone,
  landscapePhone,
  tablet,
  wideTablet,
}

extension Utility on ALDeviceType {
  /// Возвращает одно из предоставленных значений, подходящее этому
  /// типу устройства.
  T pickValue<T>({
    required T phone,
    T? landscape,
    T? tablet,
    T? wideTablet,
  }) {
    switch (this) {
      case ALDeviceType.phone:
        return phone;
      case ALDeviceType.landscapePhone:
        return landscape ?? phone;
      case ALDeviceType.tablet:
        return tablet ?? phone;
      case ALDeviceType.wideTablet:
        return wideTablet ?? tablet ?? phone;
    }
  }

  T? pickFromMap<T>(Map<ALDeviceType, T> map) {
    return pickValue(
      phone: map[ALDeviceType.phone],
      landscape: map[ALDeviceType.landscapePhone],
      tablet: map[ALDeviceType.tablet],
      wideTablet: map[ALDeviceType.wideTablet],
    );
  }

  /// Возвращает одно из предоставленных значений, подходящее этому
  /// типу устройства.
  T pickFromPT<T>(T phoneValue, T tabletValue) =>
      pickValue(phone: phoneValue, tablet: tabletValue);

  /// Возвращает одно из предоставленных значений, подходящее этому
  /// типу устройства.
  T pickFromPTW<T>(
    T phoneValue,
    T tabletValue,
    T wideTabletValue,
  ) {
    return pickValue(
      phone: phoneValue,
      tablet: tabletValue,
      wideTablet: wideTabletValue,
    );
  }
}

class ALDeviceTypeAdapter {
  static const _minTabletWidth = 600;
  static const _minWideTabletWidth = 1024;

  /// Определяет тип устройства по параметрам экрана, на котором запущено
  /// приложение.
  ///
  /// Разграничение телефонов от планшетов происходит по короткой стороне экрана,
  /// вне зависимости от ориентации. Разграничение больших и маленьких планшетов
  /// происходит по ширине экрана в текущей ориентации.
  ALDeviceType fromContext(BuildContext context) {
    final mediaQueryData = MediaQuery.of(context);
    if (mediaQueryData.size.shortestSide < _minTabletWidth) {
      return mediaQueryData.orientation != Orientation.landscape
          ? ALDeviceType.phone
          : ALDeviceType.landscapePhone;
    } else if (mediaQueryData.size.shortestSide < _minWideTabletWidth) {
      return ALDeviceType.tablet;
    } else {
      return ALDeviceType.wideTablet;
    }
  }
}
