import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';

class ALAppBar extends StatelessWidget implements PreferredSizeWidget {
  static const height = 56.0;
  static const separatorHeight = 1.0;

  final String? titleText;
  final Widget? title;
  final TextStyle? titleTextStyle;
  final List<Widget>? actions;
  final Widget? leading;
  final double? leadingWidth;
  final Color? backgroundColor;
  final IconThemeData? iconThemeData;
  final Color? statusBarColor;
  final Brightness? statusBarBrightness;
  final bool isSeparatorVisible;

  const ALAppBar({
    super.key,
    this.titleText,
    this.title,
    this.titleTextStyle,
    this.actions,
    this.leading,
    this.leadingWidth,
    this.iconThemeData,
    this.backgroundColor,
    this.statusBarColor,
    this.statusBarBrightness,
    this.isSeparatorVisible = true,
  }) : assert(
          title != null || titleText != null,
          'title or titleText must be provided',
        );

  const ALAppBar.custom({
    Key? key,
    required Widget title,
    required Widget leading,
  }) : this(
          key: key,
          titleText: null,
          title: title,
          titleTextStyle: null,
          actions: null,
          leading: leading,
          iconThemeData: null,
          isSeparatorVisible: false,
        );

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        _buildAppBar(context),
        if (isSeparatorVisible) _buildSeparator(context),
      ],
    );
  }

  Widget _buildAppBar(BuildContext context) {
    final statusBarBrightness = this.statusBarBrightness ?? context.brightness;

    return AppBar(
      title: _buildTitleWidget(context),
      actions: actions,
      leading: leading,
      leadingWidth: leadingWidth,
      iconTheme: iconThemeData,
      centerTitle: false,
      titleSpacing: leading != null ? -4 : 15,
      backgroundColor: backgroundColor ?? context.colors.whiteCard,
      systemOverlayStyle: SystemUiOverlayStyle(
        statusBarBrightness: statusBarBrightness,
        statusBarIconBrightness: statusBarBrightness.opposite,
        statusBarColor: statusBarColor ?? _getDefaultStatusBarColor(context),
      ),
      elevation: 0,
      automaticallyImplyLeading: false,
      toolbarHeight: isSeparatorVisible ? height - separatorHeight : height,
    );
  }

  Widget _buildTitleWidget(BuildContext context) {
    if (title != null) {
      return title!;
    } else {
      return Text(
        titleText!,
        style: titleTextStyle ?? context.textStyles.titleLarge,
      );
    }
  }

  Widget _buildSeparator(BuildContext context) {
    return Container(
        height: separatorHeight, color: context.colors.veryCoolGrey);
  }

  @override
  Size get preferredSize => const Size.fromHeight(height);

  Color _getDefaultStatusBarColor(BuildContext context) => Platform.isAndroid
      ? context.colors.transparent
      : context.colors.whiteCard;
}

extension _Utility on Brightness {
  Brightness get opposite {
    switch (this) {
      case Brightness.dark:
        return Brightness.light;
      case Brightness.light:
        return Brightness.dark;
    }
  }
}
