import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/extensions/color_string_extension.dart';
import 'package:altv/framework/ui/widgets/buttons/al_href_button.dart';
import 'package:altv/framework/ui/widgets/network_image/al_network_image.dart';
import 'package:flutter_widget_from_html/flutter_widget_from_html.dart';
import 'package:html/dom.dart' as dom;

/// Виджет преобразует HTML в набор стандартных виджетов.
/// Ограничения, доступные стили и теги здесь:
/// https://pub.dev/packages/flutter_widget_from_html
class ALHtmlText extends StatelessWidget {
  final String text;
  final TextStyle? textStyle;
  final CustomStylesBuilder? customStylesBuilder;
  final Function(String) onUrlTap;
  final bool useCustomImageViews;
  final bool useRoundCornersInVideo;
  final bool useRoundCornersInIframes;
  final bool useCustomHref;
  final bool replaceNewLinesWithBreakTags;
  final int? maxLines;

  const ALHtmlText(
    this.text, {
    super.key,
    required this.onUrlTap,
    this.customStylesBuilder,
    this.textStyle,
    this.useCustomImageViews = false,
    this.useRoundCornersInVideo = false,
    this.useRoundCornersInIframes = false,
    this.useCustomHref = false,
    this.replaceNewLinesWithBreakTags = true,
    this.maxLines,
  });

  ALHtmlText.basic(
    BuildContext context,
    this.text, {
    super.key,
    required this.onUrlTap,
    TextStyle? textStyle,
    this.useCustomImageViews = false,
    this.useRoundCornersInVideo = false,
    this.useRoundCornersInIframes = false,
    this.useCustomHref = false,
    this.replaceNewLinesWithBreakTags = true,
    this.maxLines,
  })  : textStyle = textStyle ?? context.textStyles.bodyRegular,
        customStylesBuilder = _makeBasicALHtmlStylesBuilder(context);

  ALHtmlText.styled(
    this.text, {
    super.key,
    required this.onUrlTap,
    required Map<String, Map<String, String>> stylesMap,
    this.textStyle,
    this.replaceNewLinesWithBreakTags = true,
    this.useCustomImageViews = false,
    this.useRoundCornersInVideo = false,
    this.useRoundCornersInIframes = false,
    this.useCustomHref = false,
    this.maxLines,
  }) : customStylesBuilder = _makeLocalNameStylesBuilder(stylesMap);

  @override
  Widget build(BuildContext context) {
    return HtmlWidget(
      _getFixedText(text),
      textStyle: textStyle ?? const TextStyle(),
      onTapUrl: (url) {
        onUrlTap(url);
        return true;
      },
      customStylesBuilder: customStylesBuilder,
      customWidgetBuilder: (element) => _buildCustomWidget(context, element),
    );
  }

  String _getFixedText(String text) {
    // Компонент не умеет обрабатывать переносы строк. Нужно заменить их на тег.
    final fixedText = text.isEmpty || !replaceNewLinesWithBreakTags
        ? text
        : text.replaceAll("\r\n", "<br>");
    return maxLines == null ? fixedText : _getTextWithMaxLinesLimit(fixedText, maxLines!);
  }

  static String _getTextWithMaxLinesLimit(String text, int maxLines) {
    return '<div style="max-lines: $maxLines; text-overflow: ellipsis">$text</div>';
  }

  static CustomStylesBuilder _makeLocalNameStylesBuilder(
      Map<String, Map<String, String>> elementStyles
  ) {
    return (element) => elementStyles[element.localName];
  }

  static CustomStylesBuilder _makeBasicALHtmlStylesBuilder(BuildContext context) {
    return (element) {
      switch (element.localName) {
        case 'a':
          return {
            'color': context.colors.blue.toHexString(),
            'font-weight': Platform.isIOS ? '600' : '500',
          };
        default:
          return null;
      }
    };
  }

  Widget? _buildCustomWidget(BuildContext context, dom.Element element) {
    if (element.localName?.toLowerCase() == "img" && useCustomImageViews) {
      return _buildCustomImageView(context, element);
    }
    if (element.localName?.toLowerCase() == "video" && useRoundCornersInVideo) {
      return _buildRoundCornersWidget(element);
    }
    if (element.localName?.toLowerCase() == "iframe" && useRoundCornersInIframes) {
      return _buildRoundCornersWidget(element);
    }
    if (_isTextHref(element) && useCustomHref) {
      return _buildCustomHref(element);
    }
    return null;
  }

  Widget? _buildCustomImageView(BuildContext context, dom.Element element) {
    final imageUrl = element.attributes["src"];
    if (imageUrl == null ||
        imageUrl.toLowerCase().startsWith("data:") ||
        Uri.tryParse(imageUrl) == null) {
      return null;
    }

    return Padding(
      padding: const EdgeInsets.only(bottom: 16),
      child: AspectRatio(
        aspectRatio: 335 / 129,
        child: ALNetworkImage(
          imageUrl: imageUrl,
          width: double.infinity,
          height: double.infinity,
          borderRadius: BorderRadius.circular(7),
          borderWidth: 1,
          borderColor: context.colors.paleGrey,
        ),
      ),
    );
  }

  Widget _buildRoundCornersWidget(dom.Element element) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 16),
      child: ClipRRect(
        borderRadius: BorderRadius.circular(10),
        child: ALHtmlText(
          element.outerHtml,
          onUrlTap: onUrlTap,
        ),
      ),
    );
  }

  bool _isTextHref(dom.Element element) {
    // Проверяем, чтобы внутри не было закрывающих тегов. Если они есть,
    // то внутри есть сложная верстка и такой элемент не подходит
    // для кастомной обработки
    return element.localName?.toLowerCase() == "a" &&
        element.attributes.containsKey("href") &&
        element.innerHtml.isNotEmpty &&
        !element.innerHtml.contains("</") &&
        !element.innerHtml.contains("/>");
  }

  Widget _buildCustomHref(dom.Element element) {
    final text = element.innerHtml;
    final href = element.attributes["href"];
    return ALHrefButton(text: text, onTap: () => onUrlTap(href ?? ""));
  }
}