import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/res/images/al_asset_change.dart';
import 'package:altv/framework/ui/widgets/buttons/al_button_icon.dart';
import 'package:altv/framework/ui/widgets/al_loading_widget.dart';

enum ALButtonType { filled, tonal, outlined, text }

class ALButton extends StatelessWidget {

  final ALButtonType type;
  final Color? color;
  final Color? splashColor;
  final Color? disabledColor;
  final String text;
  final TextStyle? titleStyle;
  final TextStyle? disabledTitleStyle;
  final Color? borderColor;
  final BorderRadius borderRadius;
  final ALButtonIcon? leftIcon;
  final ALButtonIcon? rightIcon;
  final EdgeInsets textPadding;
  final double? height;
  final double? width;
  final bool isLoading;
  final VoidCallback? onPressed;

  const ALButton({
    super.key,
    this.type = ALButtonType.filled,
    required this.onPressed,
    required this.text,
    this.titleStyle,
    this.color,
    this.splashColor,
    this.isLoading = false,
    this.disabledColor,
    this.disabledTitleStyle,
    this.borderColor,
    this.borderRadius = const BorderRadius.all(Radius.circular(8)),
    this.leftIcon,
    this.rightIcon,
    this.textPadding = EdgeInsets.zero,
    this.height = 40,
    this.width,
  });

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: height,
      width: width,
      child: _buildButton(context),
    );
  }

  Widget _buildButton(BuildContext context) {
    return TextButton(
      style: ButtonStyle(
        splashFactory: InkSplash.splashFactory,
        overlayColor: MaterialStateColor.resolveWith(
            (_) => splashColor ?? context.colors.dodgerBlue
        ),
        shape: MaterialStateProperty.resolveWith((states) =>_getBorder(context, states)),
        padding: MaterialStateProperty.all(EdgeInsets.zero),
        backgroundColor: MaterialStateColor.resolveWith((states) =>
            _getBackgroundColor(context, states)),
      ),
      onPressed: isLoading ? null : onPressed,
      child: _buildContent(context),
    );
  }

  Widget _buildContent(BuildContext context) {
    if (isLoading) {
      return Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          ALLoadingWidget(
            color: _getTextColorByType(context),
            size: 20,
          )
        ],
      );
    } else {
      return Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          _buildIcon(
            context,
            leftIcon,
            leftIcon?.padding ?? const EdgeInsets.only(right: 8),
          ),
          Flexible(
            child: Padding(
              padding: textPadding,
              child: Text(
                text,
                textAlign: TextAlign.center,
                style: onPressed != null
                    ? titleStyle ?? _getDefaultTextStyleByType(context)
                    : disabledTitleStyle ?? _getDefaultDisabledTextStyleByType(context),
                overflow: TextOverflow.ellipsis,
              ),
            ),
          ),
          _buildIcon(
            context,
            rightIcon,
            rightIcon?.padding ?? const EdgeInsets.only(left: 8),
          ),
        ],
      );
    }
  }

  Widget _buildIcon(
      BuildContext context,
      ALButtonIcon? buttonIcon,
      EdgeInsets padding,
  ) {
    return buttonIcon == null ? const SizedBox.shrink() : Padding(
      padding: padding,
      child: buttonIcon.icon.buildWidget(
        context,
        color: buttonIcon.color ?? _getTextColorByType(context),
        change: ALAssetChange.square(buttonIcon.size ?? 18),
      ),
    );
  }

  // Цвета и стили по типу кнопки

  OutlinedBorder _getBorder(BuildContext context, Set<MaterialState> states) {
    final border = type == ALButtonType.outlined
        ? BorderSide(width: 1, color: borderColor ?? _getBorderColor(context, states))
        : BorderSide.none;
    return RoundedRectangleBorder(
      borderRadius: borderRadius,
      side: border,
    );
  }

  Color _getBackgroundColor(BuildContext context, Set<MaterialState> states) {
    final backgroundColor = _getBackgroundColorByType(context);
    final backgroundDisabledColor = isLoading
        ? backgroundColor
        : _getBackgroundDisabledColorByType(context);
    return states.contains(MaterialState.disabled)
        ? backgroundDisabledColor
        : backgroundColor;
  }

  Color _getBorderColor(BuildContext context, Set<MaterialState> states) {
    final backgroundColor = _getBorderColorByType(context);
    final backgroundDisabledColor = isLoading
        ? backgroundColor
        : _getBorderDisabledColorByType(context);
    return states.contains(MaterialState.disabled)
        ? backgroundDisabledColor
        : backgroundColor;
  }

  Color _getBackgroundColorByType(BuildContext context) {
    switch (type) {
      case ALButtonType.filled:
        return color ?? context.colors.primary;
      case ALButtonType.tonal:
        return color ?? context.colors.lightIndigo;
      case ALButtonType.outlined:
      case ALButtonType.text:
        return context.colors.transparent;
    }
  }

  Color _getBackgroundDisabledColorByType(BuildContext context) {
    switch (type) {
      case ALButtonType.filled:
      case ALButtonType.tonal:
        return disabledColor ?? context.colors.paleGreyDisabled;
      case ALButtonType.outlined:
      case ALButtonType.text:
        return context.colors.transparent;
    }
  }

  TextStyle _getDefaultTextStyleByType(BuildContext context) {
    return context.textStyles.bodyBold.copyWith(color: _getTextColorByType(context));
  }

  Color _getTextColorByType(BuildContext context) {
    switch (type) {
      case ALButtonType.filled:
      case ALButtonType.tonal:
        return context.colors.white;
      case ALButtonType.outlined:
      case ALButtonType.text:
        return color ?? context.colors.primary;
    }
  }

  TextStyle _getDefaultDisabledTextStyleByType(BuildContext context) {
    final Color textColor;
    switch (type) {
      case ALButtonType.filled:
      case ALButtonType.tonal:
      case ALButtonType.outlined:
      case ALButtonType.text:
        textColor = context.colors.slateGrey;
    }
    return context.textStyles.bodyBold.copyWith(color: textColor);
  }

  Color _getBorderColorByType(BuildContext context) {
    switch (type) {
      case ALButtonType.outlined:
        return color ?? context.colors.primary;
      case ALButtonType.filled:
      case ALButtonType.tonal:
      case ALButtonType.text:
        return context.colors.transparent;
    }
  }

  Color _getBorderDisabledColorByType(BuildContext context) {
    switch (type) {
      case ALButtonType.outlined:
        return disabledColor ?? context.colors.paleGreyDisabled;
      case ALButtonType.filled:
      case ALButtonType.tonal:
      case ALButtonType.text:
        return context.colors.transparent;
    }
  }
}
