import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/res/images/al_image_asset.dart';
import 'package:altv/framework/theme/al_theme_colors.dart';

enum ALChipsTabBarItemType { text, icon }

class ALChipsTabBarItem {
  final ALChipsTabBarItemType type;
  final String value;

  ALChipsTabBarItem(this.type, this.value);
}

/// Панель для чипсов - фишек выбора, представляющих набор заданных вариантов
/// с возможностью выбора одного из них
class ALChipsTabBar extends StatefulWidget {
  final List<ALChipsTabBarItem> items;
  final TabController controller;
  final Color backgroundColor;
  final Color defaultColor;
  final Color selectedColor;
  final Color defaultTextColor;
  final Color selectedTextColor;
  final Function(int)? onSelectItem;
  final EdgeInsets padding;

  final double iconSize = 16;

  ALChipsTabBar({
    super.key,
    required BuildContext context,
    required this.items,
    required this.controller,
    Color? backgroundColor,
    Color? defaultColor,
    Color? selectedColor,
    Color? defaultTextColor,
    Color? selectedTextColor,
    this.padding = const EdgeInsets.only(top: 14.0, bottom: 5.0),
    this.onSelectItem,
  })  : backgroundColor = backgroundColor ?? context.colors.white,
        defaultColor = defaultColor ?? context.colors.paleGrey,
        selectedColor = selectedColor ?? context.colors.primary,
        defaultTextColor = defaultTextColor ?? context.colors.slateGrey,
        selectedTextColor = selectedTextColor ?? ALThemeColors.lightColors.white;

  @override
  State<ALChipsTabBar> createState() => _ALChipsTabBarState();
}

class _ALChipsTabBarState extends State<ALChipsTabBar> {
  // Индекс для которого отрисовывается выделение
  int? _selectedIndex;

  @override
  void initState() {
    _selectedIndex = widget.controller.index;
    // Подключаемся к листнеру анимации
    // что бы при свайпе по странице обеспечить переключение чипсов без задержки

    widget.controller.animation?.addListener(_animationListener);
    super.initState();
  }

  @override
  void dispose() {
    widget.controller.animation?.removeListener(_animationListener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _disableRippleEffect(
      child: Container(
        padding: widget.padding,
        color: widget.backgroundColor,
        child: Align(
          alignment: Alignment.bottomLeft,
          child: IntrinsicHeight(
            child: TabBar(
              controller: widget.controller,
              tabs: _tabsBuilder(context),
              indicator: const BoxDecoration(),
              indicatorWeight: 0,
              isScrollable: true,
              labelPadding: EdgeInsets.zero,
              onTap: _onTapItem,
            ),
          ),
        ),
      ),
    );
  }

  Widget _disableRippleEffect({required Widget child}) {
    return Theme(
      data: ThemeData(
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
      ),
      child: child,
    );
  }

  void _animationListener() {
    final animation = widget.controller.animation;
    if (animation == null) {
      return;
    }
    // Используем округленный индекс из анимации
    // что бы при свайпе по странице переключение чипсов происходило без задержки
    final int currentAnimationIndex = animation.value.round();
    // Условие для исключения лишней перерисовки при прохождении анимацией промежуточных табов
    if (_selectedIndex != currentAnimationIndex &&
        (_selectedIndex == widget.controller.index ||
            currentAnimationIndex == widget.controller.index)) {
      setState(() {
        _selectedIndex = currentAnimationIndex;
      });
    }
  }

  List<Widget> _tabsBuilder(BuildContext context) {
    return widget.items
        .asMap()
        .map((index, item) => MapEntry(index, _tabBuilder(context, index)))
        .values
        .toList();
  }

  Widget _tabBuilder(BuildContext context, int index) {
    final isFirtsItem = index == 0;
    final isLastItem = index == widget.items.length - 1;

    double leftPadding = isFirtsItem ? 20 : 3;
    double rightPadding = isLastItem ? 20 : 3;

    return Padding(
      padding: EdgeInsets.only(left: leftPadding, right: rightPadding),
      child: ClipRRect(
        borderRadius: const BorderRadius.all(
          Radius.circular(8.0),
        ),
        child: Container(
          alignment: Alignment.center,
          color: _getBackgroundColor(index),
          child: _chipsItem(index),
        ),
      ),
    );
  }

  Widget _chipsItem(int index) {
    return widget.items[index].type == ALChipsTabBarItemType.text
        ? _textItem(index)
        : _iconItem(index);
  }

  Widget _textItem(int index) {
    return Padding(
      padding: const EdgeInsets.only(left: 9, right: 9, bottom: 3),
      child: Text(
        widget.items[index].value,
        style: context.textStyles.bodyMedium.copyWith(
          color: _getTextColor(index),
        ),
      ),
    );
  }

  Widget _iconItem(int index) {
    final iconName = widget.items[index].value;
    return Container(
      alignment: Alignment.center,
      width: 32,
      child: ALImageAsset.vector(
          iconName,
          size: Size.square(widget.iconSize),
      ).buildWidget(context, color: _getTextColor(index)),
    );
  }

  Color _getTextColor(int index) {
    return index == _selectedIndex
        ? widget.selectedTextColor
        : widget.defaultTextColor;
  }

  Color _getBackgroundColor(int index) {
    return index == _selectedIndex ? widget.selectedColor : widget.defaultColor;
  }

  void _onTapItem(int index) {
    widget.onSelectItem?.call(index);
  }
}
