import 'dart:ui' as ui show BoxHeightStyle, BoxWidthStyle;

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';

/// Поле ввода с отслеживанием ввода текста и передаваемым обработчиком фокуса.
/// Является частью поля ввода input_text.dart, input_number.dart

class ALInputText extends StatelessWidget {
  final String? title;
  final String? hintText;
  final String? errorText;
  final bool enabled;
  final FocusNode? focusNode;
  final TextInputType inputType;
  final TextAlign textAlign;
  final TextEditingController controller;
  final Function(String)? onSubmitted;
  final List<TextInputFormatter>? inputFormatters;
  final Widget? leftWidget;
  final Widget? rightWidget;
  final EdgeInsets leftWidgetPadding;
  final EdgeInsets rightWidgetPadding;
  final double? leftWidgetWidth;
  final double? rightWidgetWidth;
  final Function()? onClickLeftWidget;
  final Function()? onClickRightWidget;
  final int? maxLength;
  final int? maxLines;
  final int? minLines;
  final bool expands;
  final bool autofocus;
  final String obscuringCharacter;
  final bool obscureText;
  final bool autocorrect;
  final bool enableSuggestions;
  final TextAlignVertical? textAlignVertical;
  final TextInputAction? textInputAction;
  final InputDecoration? inputDecoration;
  final TextCapitalization textCapitalization;
  final bool readOnly;
  final double cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final ui.BoxHeightStyle selectionHeightStyle;
  final ui.BoxWidthStyle selectionWidthStyle;
  final Brightness? keyboardAppearance;
  final EdgeInsets scrollPadding;
  final bool enableInteractiveSelection;
  final TextSelectionControls? selectionControls;
  final DragStartBehavior dragStartBehavior;
  final GestureTapCallback? onTap;
  final MouseCursor? mouseCursor;
  final ScrollPhysics? scrollPhysics;
  final ScrollController? scrollController;
  final Iterable<String>? autofillHints;

  /// [leftWidgetWidth] - нужно явно задавать ширину иконки для корректных отступов текста слева, в силу реализации через Stack
  /// [rightWidgetWidth] - нужно явно задавать ширину иконки для корректных отступов текста слева, в силу реализации через Stack
  /// [errorTextBackgroundColor] - цвет фона текста ошибки, если не задать, то ошибка будет отображаться без фона
  /// [fieldFillColor] - цвет заливки фона текстового поля, по умолчанию прозрачный
  /// [borderEnabledColor] - цвет рамки в обычном состоянии при enabled = true
  /// [borderFocusedColor] - цвет рамки когда есть фокус на поле
  const ALInputText({
    super.key,
    required this.controller,
    this.title,
    this.hintText,
    this.errorText,
    this.focusNode,
    this.inputFormatters,
    this.leftWidget,
    this.rightWidget,
    this.leftWidgetPadding = const EdgeInsets.only(left: 20, right: 5),
    this.rightWidgetPadding = const EdgeInsets.only(left: 5, right: 16),
    this.leftWidgetWidth,
    this.rightWidgetWidth,
    this.onClickLeftWidget,
    this.onClickRightWidget,
    this.inputType = TextInputType.text,
    this.textAlign = TextAlign.left,
    this.enabled = true,
    this.onSubmitted,
    this.maxLength,
    this.maxLines,
    this.minLines = 1,
    this.expands = false,
    this.autofocus = false,
    this.obscuringCharacter = '•',
    this.obscureText = false,
    this.autocorrect = true,
    this.enableSuggestions = true,
    this.textAlignVertical,
    this.textInputAction,
    this.inputDecoration,
    this.textCapitalization = TextCapitalization.none,
    this.readOnly = false,
    this.cursorWidth = 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.selectionHeightStyle = ui.BoxHeightStyle.tight,
    this.selectionWidthStyle = ui.BoxWidthStyle.tight,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.dragStartBehavior = DragStartBehavior.start,
    this.enableInteractiveSelection = true,
    this.selectionControls,
    this.onTap,
    this.mouseCursor,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
  });

  String get _counterText {
    if ((maxLength ?? 0) > 0) {
      final textLength = controller.value.text.characters.length;
      return '$textLength/$maxLength';
    } else {
      return '';
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.max,
      children: [
        _buildTitle(context),
        _buildTextInputLayout(context),
        _buildErrorText(context),
        _buildCounter(context),
      ],
    );
  }

  Widget _buildTitle(BuildContext context) {
    return title != null
        ? _buildInfoTitle(text: title!, textStyle: _getTitleStyle(context))
        : Container();
  }

  Widget _buildErrorText(BuildContext context) {
    return errorText != null
        ? _buildInfoTitle(
            text: errorText!,
            textStyle: _getErrorStyle(context),
            backgroundColor: _getErrorTextBackgroundColor(context))
        : Container();
  }

  Widget _buildCounter(BuildContext context) {
    return maxLength != null
        ? _buildInfoTitle(
            text: _counterText,
            textStyle: _getCounterStyle(context),
          )
        : Container();
  }

  Widget _buildTextInputLayout(BuildContext context) {
    return Stack(children: [
      _buildTextField(context),
      Positioned.fill(
        child: Align(
          alignment: Alignment.centerLeft,
          child: _buildLeftIcon(),
        ),
      ),
      Positioned.fill(
        child: Align(
          alignment: Alignment.centerRight,
          child: _buildRightIcon(),
        ),
      ),
    ]);
  }

  Widget _buildTextField(BuildContext context) {
    return TextField(
      enabled: enabled,
      textDirection: TextDirection.ltr,
      onSubmitted: onSubmitted,
      textAlign: textAlign,
      keyboardType: inputType,
      inputFormatters: inputFormatters,
      style: _getInputTextStyle(context),
      cursorColor: _getCursorColor(context),
      controller: controller,
      focusNode: focusNode,
      minLines: minLines,
      maxLength: maxLength,
      maxLines: maxLines,
      expands: expands,
      autofocus: autofocus,
      obscuringCharacter: obscuringCharacter,
      obscureText: obscureText,
      autocorrect: autocorrect,
      enableSuggestions: enableSuggestions,
      textAlignVertical: textAlignVertical,
      textInputAction: textInputAction,
      decoration: _buildFieldDecoration(context),
      textCapitalization: textCapitalization,
      readOnly: readOnly,
      cursorWidth: cursorWidth,
      cursorHeight: cursorHeight,
      cursorRadius: cursorRadius,
      selectionHeightStyle: selectionHeightStyle,
      selectionWidthStyle: selectionWidthStyle,
      keyboardAppearance: keyboardAppearance,
      scrollPadding: scrollPadding,
      dragStartBehavior: dragStartBehavior,
      enableInteractiveSelection: enableInteractiveSelection,
      selectionControls: selectionControls,
      onTap: onTap,
      mouseCursor: mouseCursor,
      scrollController: scrollController,
      scrollPhysics: scrollPhysics,
      autofillHints: autofillHints,
    );
  }

  Widget _buildLeftIcon() {
    if (leftWidget == null) {
      return Container();
    }
    return GestureDetector(
      onTap: onClickLeftWidget,
      child: Padding(
        padding: leftWidgetPadding,
        child: leftWidget,
      ),
    );
  }

  Widget _buildRightIcon() {
    if (rightWidget == null) {
      return Container();
    }
    return GestureDetector(
      onTap: onClickRightWidget,
      child: Padding(
        padding: rightWidgetPadding,
        child: rightWidget,
      ),
    );
  }

  Widget _buildInfoTitle(
      {required String text,
      required TextStyle textStyle,
      Color? backgroundColor}) {
    return Align(
      alignment: Alignment.centerLeft,
      child: _ALTitleMessage(
        text: text,
        textStyle: textStyle,
        backgroundColor: backgroundColor,
      ),
    );
  }

  InputDecoration _buildFieldDecoration(BuildContext context) {
    if (inputDecoration == null) {
      double leftIconTotalPadding = leftWidget == null
          ? 0
          : (leftWidgetPadding.left + leftWidgetPadding.right);
      double rightIconTotalPadding = rightWidget == null
          ? 0
          : (rightWidgetPadding.left + rightWidgetPadding.right);
      double leftPadding = 20 + leftIconTotalPadding + (leftWidgetWidth ?? 0);
      double rightPadding =
          20 + rightIconTotalPadding + (rightWidgetWidth ?? 0);
      return InputDecoration(
        // Мы скрываем библиотечный счетчик длины текста, потому что его расположение
        // намертво привязано к contentPadding'у
        counterText: '',
        contentPadding: EdgeInsets.only(
          left: leftPadding,
          top: 20,
          right: rightPadding,
          bottom: 23,
        ),
        fillColor: _getFieldFillColor(context),
        filled: true,
        hintStyle: _getHintStyle(context),
        hintText: hintText,
        enabledBorder: _buildBorder(_getBorderEnabledColor(context)),
        disabledBorder: _buildBorder(_getBorderDisabledColor(context)),
        focusedBorder: _buildBorder(_getBorderFocusedColor(context)),
      );
    } else {
      return inputDecoration!;
    }
  }

  OutlineInputBorder _buildBorder(Color color) {
    return OutlineInputBorder(
      borderRadius: BorderRadius.circular(10),
      borderSide: BorderSide(
        color: color,
        width: 1,
      ),
    );
  }

  Color _getErrorTextBackgroundColor(BuildContext context) =>
      context.colors.lightPink;
  Color _getCursorColor(BuildContext context) => context.colors.dark;
  Color _getFieldFillColor(BuildContext context) => context.colors.transparent;
  Color _getBorderEnabledColor(BuildContext context) =>
      context.colors.veryCoolGrey;
  Color _getBorderDisabledColor(BuildContext context) =>
      context.colors.veryCoolGrey;
  Color _getBorderFocusedColor(BuildContext context) => context.colors.coolGrey;

  TextStyle _getTitleStyle(BuildContext context) =>
      context.textStyles.labelLargeMedium;
  TextStyle _getInputTextStyle(BuildContext context) =>
      context.textStyles.titleSmallMedium;
  TextStyle _getHintStyle(BuildContext context) =>
      context.textStyles.titleSmallMedium
          .copyWith(color: context.colors.coolGrey);
  TextStyle _getErrorStyle(BuildContext context) =>
      context.textStyles.bodySmallMedium.copyWith(color: context.colors.orange);
  TextStyle _getCounterStyle(BuildContext context) =>
      context.textStyles.bodySmallMedium;
}

class _ALTitleMessage extends StatelessWidget {
  final TextStyle textStyle;
  final Color? backgroundColor;
  final String text;

  const _ALTitleMessage({
    required this.text,
    required this.textStyle,
    this.backgroundColor,
  });

  @override
  Widget build(BuildContext context) {
    return Visibility(
      visible: text.isNotEmpty,
      child: Padding(
        padding: const EdgeInsets.only(
          bottom: 7,
          top: 5,
        ),
        child: backgroundColor == null
            ? _buildTitle()
            : _buildTitleWithBackground(),
      ),
    );
  }

  Widget _buildTitleWithBackground() {
    return Container(
      padding: const EdgeInsets.only(
        left: 15,
        right: 15,
        bottom: 2,
        top: 2,
      ),
      decoration: ShapeDecoration(
        color: backgroundColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(6),
        ),
      ),
      child: _buildTitle(),
    );
  }

  Widget _buildTitle() {
    return Text(text, style: textStyle);
  }
}
