import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/ui/widgets/al_shimmer.dart';
import 'package:altv/framework/ui/widgets/network_image/network_image_no_cache/logic/bloc/al_network_image_no_cache_cubit.dart';
import 'package:altv/framework/ui/widgets/network_image/network_image_no_cache/ui/al_network_image_no_cache_widget.dart';
import 'package:altv/main.dart';
import 'package:jovial_svg/jovial_svg.dart';

class ALNetworkImage extends StatelessWidget {
  static const defaultSizeLimit = Size.square(2048);

  final String imageUrl;
  final double? width;
  final double? height;
  final BorderRadius borderRadius;
  final double borderWidth;
  final Color? borderColor;

  final List<BoxShadow> shadows;
  final BoxFit fit;

  final Widget Function(BuildContext)? errorWidget;
  final Widget Function(BuildContext)? placeholder;
  final Widget Function(BuildContext, Widget)? imageBuilder;

  final Size sizeLimit;

  final bool isSvgSupported;
  final Uri? baseUri;

  /// Свойство, указывающее какой внутренний виджет будет использован для изображения.
  /// Используется изображение сохраняющееся в кеш, либо изображение без кеша.
  final bool isCacheEnabled;

  /// Свойство-индикатор для некеширующего изображения. Для индикации того, необходимо ли перезагрузить контент.
  final bool isReloadRequired;
  final Function()? onReloadInvoked;

  const ALNetworkImage({
    super.key,
    required this.imageUrl,
    this.width,
    this.height,
    this.borderRadius = BorderRadius.zero,
    this.borderWidth = 0,
    this.borderColor,
    this.shadows = const [],
    this.fit = BoxFit.cover,
    this.errorWidget,
    this.placeholder,
    this.imageBuilder,
    this.sizeLimit = defaultSizeLimit,
    this.isSvgSupported = false,
    this.baseUri,
    this.isCacheEnabled = true,
    this.isReloadRequired = false,
    this.onReloadInvoked,
  });

  ALNetworkImage.round({
    super.key,
    required this.imageUrl,
    required double size,
    this.borderWidth = 0,
    this.borderColor,
    this.shadows = const [],
    this.fit = BoxFit.cover,
    this.errorWidget,
    this.placeholder,
    this.imageBuilder,
    this.sizeLimit = defaultSizeLimit,
    this.isSvgSupported = false,
    this.baseUri,
    this.isCacheEnabled = true,
    this.isReloadRequired = false,
    this.onReloadInvoked,
  })  : width = size,
        height = size,
        borderRadius = BorderRadius.circular(size / 2 - borderWidth);

  @override
  Widget build(BuildContext context) {
    return _buildFraming(
      context: context,
      child: _buildContent(context),
    );
  }

  Widget _buildContent(BuildContext context) {
    final imageUri = baseUri?.resolve(imageUrl) ?? Uri.parse(imageUrl);

    if (isSvgSupported && imageUrl.endsWith('svg')) {
      // TODO: Когда flutter_svg добавит адекватную работу с ошибками загрузки,
      // заменить код ниже на реализацию с его помощью. Тогда из проекта можно
      // будет убрать jovial_svg, и у нас не будет использоваться два плагина
      // для работы с svg.
      // https://github.com/dnfield/flutter_svg/issues/921
      // https://github.com/dnfield/flutter_svg/issues/996

      return ScalableImageWidget.fromSISource(
        fit: fit,
        si: ScalableImageSource.fromSvgHttpUrl(imageUri),
        onLoading: placeholder ?? _defaultPlaceholder,
        onError: errorWidget ?? _defaultErrorWidget,
      );
    } else if (isCacheEnabled) {
      return CachedNetworkImage(
        imageUrl: imageUri.toString(),
        fadeInDuration: const Duration(milliseconds: 300),
        placeholder: (context, _) =>
            (placeholder ?? _defaultPlaceholder).call(context),
        errorWidget: (context, _, __) =>
            (errorWidget ?? _defaultErrorWidget).call(context),
        imageBuilder: (context, imageProvider) {
          final imageWidget = _buildImageWithProvider(imageProvider);
          return (imageBuilder ?? _defaultImageBuilder)
              .call(context, imageWidget);
        },
        maxWidthDiskCache: sizeLimit.width.toInt(),
        maxHeightDiskCache: sizeLimit.height.toInt(),
      );
    } else {
      return BlocProvider<ALNetworkImageNoCacheCubit>(
        create: (context) => diContainer.resolve<ALNetworkImageNoCacheCubit>(),
        child: ALNetworkImageNoCacheWidget(
          url: imageUri.toString(),
          placeholder: (context) => (placeholder ?? _defaultPlaceholder).call(context),
          errorWidget: (errorWidget ?? _defaultErrorWidget).call(context),
          imageBuilder: _buildImageFromBytes,
          isReloadRequired: isReloadRequired,
          onReloadInvoked: onReloadInvoked,
        ),
      );
    }
  }

  Widget _buildImageWithProvider(ImageProvider imageProvider) {
    return Image(
      width: double.infinity,
      height: double.infinity,
      image: imageProvider,
      fit: fit,
    );
  }

  Widget _defaultImageBuilder(BuildContext context, Widget imageWidget) {
    return imageWidget;
  }

  Widget _defaultErrorWidget(BuildContext context) {
    return const SizedBox.shrink();
  }

  Widget _defaultPlaceholder(BuildContext context) {
    return ALShimmer(
      context: context,
      child: Container(color: context.colors.skeleton),
    );
  }

  Widget _buildFraming({required BuildContext context, required Widget child}) {
    return Container(
      decoration: BoxDecoration(
        borderRadius: borderRadius.add(BorderRadius.circular(borderWidth)),
        boxShadow: shadows,
        border: Border.all(
          style: BorderStyle.solid,
          width: borderWidth,
          color: borderColor ?? context.colors.transparent,
        ),
      ),
      width: width,
      height: height,
      child: ClipRRect(
        borderRadius: borderRadius,
        child: child,
      ),
    );
  }

  Widget _buildImageFromBytes(BuildContext context, Uint8List imageInfo) {
    return Image.memory(
      imageInfo,
      width: double.infinity,
      height: double.infinity,
      fit: fit,
    );
  }
}
