import 'package:flutter/material.dart';
import 'package:altv/framework/errors/error_handler/error_handler.dart';
import 'package:altv/framework/ui/widgets/paging/al_paging_list_view_paging_footer.dart';
import 'package:altv/framework/ui/widgets/paging/al_paging_list_view_reloading_header.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

/// Перечисление возможных состояний списка [ALPagingListView]
enum ALPagingListViewState {
  emptyInitial,
  loadInProgress,
  loadSuccess,
  loadFailure,
  reloadInProgress,
  reloadSuccess,
  reloadFailure,
  pagingInProgress,
  pagingSuccess,
  pagingFailure,
}

/// Виджет для осуществления постраничной загрузки,
/// и обновления с помощью PullToRefresh
///
/// [state] текущее состояние [ALPagingListViewState] списка
///
/// [onReload] делегат перезагрузки
///
/// [onLoadingNextPage] делегат загрузки следующей страницы
///
/// [itemBuilder] метод построения следующего элемента списка
///
/// [itemCount] количество элементов в списке
///
/// [skeletonBuilder] делегат построения скелетона. Ожидается что вернёт готовый список виджетов
///
/// [emptyListBuilder] делегат построения виджета, символизирующего загруженный пустой список
///
/// [failureReason] ошибка, которую необходимо отобразить на экране
///
/// [headerIndicatorMargin] отступы вокруг верхнего индикатора загрузки, которые отделяют его
/// сверху от границы выделенного пространства и снизу от элементов списка.
///
/// [footerIndicatorMargin] отступы вокруг нижнего индикатора загрузки, которые отделяют его
/// сверху от элементов списка и снизу от границы выделенного пространства.
///
/// [shrinkWrap] флаг, который определяет поведение виджета списка относительно
/// вертикального пространства. Если ```true```, виджет будет сжиматься до
/// размеров контента. Если ```false```, виджет будет стараться растянуться на
/// все доступное пространство.
class ALPagingListView extends StatelessWidget {
  final ALPagingListViewState state;
  final ErrorHandler errorHandler;
  final RefreshController controller;
  final ScrollController? scrollController;
  final Function() onLoad;
  final Function() onReload;
  final Function() onLoadingNextPage;
  final IndexedWidgetBuilder itemBuilder;
  final IndexedWidgetBuilder? separatorBuilder;
  final int itemCount;
  final bool hasNextPage;
  final Widget Function() skeletonBuilder;
  final Widget Function() emptyListBuilder;
  final Exception? failureReason;
  final EdgeInsets? listPadding;
  final EdgeInsets headerIndicatorMargin;
  final EdgeInsets footerIndicatorMargin;
  final bool shrinkWrap;
  final bool enablePullDown;
  final Widget? header;
  final Widget? footer;

  const ALPagingListView({
    super.key,
    required this.state,
    required this.errorHandler,
    required this.controller,
    this.scrollController,
    required this.onLoad,
    required this.onReload,
    required this.onLoadingNextPage,
    required this.itemBuilder,
    required this.itemCount,
    required this.hasNextPage,
    required this.skeletonBuilder,
    required this.emptyListBuilder,
    required this.failureReason,
    this.separatorBuilder,
    this.listPadding,
    this.headerIndicatorMargin = const EdgeInsets.only(top: 16.0),
    this.footerIndicatorMargin = const EdgeInsets.only(bottom: 24.0, top: 24.0),
    this.shrinkWrap = false,
    this.enablePullDown = true,
    this.header,
    this.footer,
  });

  @override
  Widget build(BuildContext context) {
    switch (state) {
      case ALPagingListViewState.reloadSuccess:
        controller.refreshCompleted();
        break;
      case ALPagingListViewState.reloadFailure:
        controller.refreshFailed();
        break;
      case ALPagingListViewState.pagingSuccess:
        controller.loadComplete();
        break;
      case ALPagingListViewState.pagingFailure:
        controller.loadFailed();
        break;
      default:
    }

    if (state == ALPagingListViewState.loadFailure) {
      return errorHandler.buildErrorWidget(
        exception: failureReason ?? Exception(),
        onRetry: onLoad,
      );
    } else if (state == ALPagingListViewState.emptyInitial ||
        state == ALPagingListViewState.loadInProgress) {
      return skeletonBuilder();
    } else if (itemCount == 0 && state == ALPagingListViewState.reloadInProgress) {
      return skeletonBuilder();
    } else {
      return _buildRefresher(context);
    }
  }

  Widget _buildRefresher(BuildContext context) {
    return Scrollbar(
      child: SmartRefresher(
        controller: controller,
        enablePullDown: enablePullDown,
        enablePullUp: hasNextPage,
        scrollController: scrollController,
        header: header ?? ALPagingListViewReloadingHeader(margin: headerIndicatorMargin),
        footer: footer ?? ALPagingListViewPagingFooter(margin: footerIndicatorMargin),
        onRefresh: onReload,
        onLoading: onLoadingNextPage,
        child: _buildList(),
      ),
    );
  }

  Widget _buildList() {
    if (itemCount > 0) {
      return ListView.separated(
        shrinkWrap: shrinkWrap,
        padding: listPadding,
        physics: const AlwaysScrollableScrollPhysics(),
        itemCount: itemCount,
        itemBuilder: itemBuilder,
        separatorBuilder: (context, index) {
          return separatorBuilder != null
              ? separatorBuilder!(context, index)
              : const SizedBox.shrink();
        },
      );
    } else {
      return emptyListBuilder();
    }
  }
}
