import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/ui/widgets/buttons/al_button.dart';
import 'package:altv/framework/ui/widgets/al_html_text.dart';
import 'package:altv/framework/ui/widgets/popup/al_modal_popup.dart';
import 'package:intersperse/intersperse.dart';

/// Расположение кнопок в попапе
enum ALAlertPopupButtonLayout {
  /// Кнопки располагаются в одной строке
  horizontal,

  /// Кнопки располагаются друг под другом
  vertical,
}

/// Показывает попап с заголовком, текстом и кнопками
Future<T?> showALAlertPopup<T>({
  required BuildContext context,
  required String? text,
  required List<ALAlertPopupButton> buttons,
  void Function(String)? onHtmlUrlTap,
  String? title,
  TextAlign textAlign = TextAlign.center,
  bool isDismissible = true,
  ALAlertPopupButtonLayout buttonsLayout = ALAlertPopupButtonLayout.horizontal,
  bool isHtmlText = false,
  // Сюда можно добавить произвольный виджет, который будет располагаться между текстом и кнопками
  Widget? additionalContent,
  // Включает нижний отступ, чтобы попап не перекрывался клавиатурой
  bool viewInsetsPaddingEnabled = false,
}) {
  return showALModalPopup(
    context: context,
    isDismissible: isDismissible,
    enableDrag: isDismissible,
    isViewInsetsPaddingEnabled: viewInsetsPaddingEnabled,
    body: PRAlertPopupContentWidget(
      title: title,
      text: text,
      isHtmlText: isHtmlText,
      onHtmlUrlTap: onHtmlUrlTap,
      textAlign: textAlign,
      buttons: buttons,
      buttonsLayout: buttonsLayout,
      additionalContent: additionalContent,
    ),
  );
}

class ALAlertPopupButton extends StatelessWidget {
  final String text;
  final ALButtonType buttonType;
  final Function()? onPressed;

  /// Флаг, отмечающий кнопку как скрывающую попап или не скрывающую.
  ///
  /// Значение true этого флага означает, что перед выполнением [onPressed],
  /// навигатору текущего контекста будет дана команда закрыть самый внешний
  /// элемент навигации (т.е. скрыть попап). Чаще всего, это желаемое и
  /// ожидаемое поведение.
  ///
  /// В тех редких случаях, когда по какой-либо причине
  /// мы хотим произвести скрытие попапа самостоятельно, этому свойству можно
  /// задать значение false.
  final bool isDismissing;

  const ALAlertPopupButton({
    super.key,
    this.buttonType = ALButtonType.filled,
    required this.text,
    required this.onPressed,
    this.isDismissing = true,
  });

  @override
  Widget build(BuildContext context) {
    void onPressed() => _onButtonPressed(context);
    return ALButton(text: text, onPressed: onPressed, type: buttonType,);
  }

  void _onButtonPressed(BuildContext context) {
    if (isDismissing) {
      Navigator.of(context).pop();
    }

    onPressed?.call();
  }
}

class PRAlertPopupContentWidget extends StatelessWidget {
  final TextAlign textAlign;
  final String? title;
  final String? text;
  final List<ALAlertPopupButton> buttons;
  final ALAlertPopupButtonLayout buttonsLayout;
  final bool isHtmlText;
  final Widget? additionalContent;
  final Function(String)? onHtmlUrlTap;

  const PRAlertPopupContentWidget({
    super.key,
    required this.text,
    required this.buttons,
    this.onHtmlUrlTap,
    this.title,
    this.isHtmlText = false,
    this.textAlign = TextAlign.center,
    this.buttonsLayout = ALAlertPopupButtonLayout.vertical,
    this.additionalContent,
  });

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      mainAxisSize: MainAxisSize.min,
      children: [
        _buildTitle(context),
        _buildText(context),
        _buildAdditionalContent(),
        _buildButtons(),
      ],
    );
  }

  Widget _buildTitle(BuildContext context) {
    final isTextVisible = text != null && text!.isNotEmpty;
    return Visibility(
      visible: title != null && title!.isNotEmpty,
      child: Padding(
        padding: EdgeInsets.fromLTRB(32, 0, 49, isTextVisible ? 12 : 8),
        child: Text(
          title ?? "",
          style: context.textStyles.headlineSmallBold,
          textAlign: textAlign,
        ),
      ),
    );
  }

  Widget _buildText(BuildContext context) {
    return Visibility(
      visible: text != null && text!.isNotEmpty,
      child: Padding(
        padding: const EdgeInsets.fromLTRB(32, 0, 49, 8),
        child: isHtmlText
            ? ALHtmlText(text ?? "", onUrlTap: onHtmlUrlTap ?? (_) {})
            : Text(
                text ?? "",
                style: context.textStyles.bodyRegular,
                textAlign: textAlign,
              ),
      ),
    );
  }

  Widget _buildAdditionalContent() {
    return Visibility(
      visible: additionalContent != null,
      child: Padding(
        padding: const EdgeInsets.fromLTRB(20, 0, 20, 8),
        child: additionalContent,
      ),
    );
  }

  Widget _buildButtons() {
    return Visibility(
      visible: buttons.isNotEmpty,
      child: buttonsLayout == ALAlertPopupButtonLayout.horizontal
          ? _buildHorizontalButtons()
          : _buildVerticalButtons(),
    );
  }

  Widget _buildHorizontalButtons() {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16.0, 16.0, 16.0, 0.0),
      child: Row(
        children: buttons
            .map<Widget>((b) => Expanded(child: b))
            .intersperse(const SizedBox(width: 12))
            .toList(),
      ),
    );
  }

  Widget _buildVerticalButtons() {
    return Padding(
      padding: const EdgeInsets.fromLTRB(20.0, 16.0, 20.0, 0.0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: buttons
            .map<Widget>((b) => b)
            .intersperse(const SizedBox(height: 8))
            .toList(),
      ),
    );
  }
}
