import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';

/// Показывает попап, высота которого подстраивается под содержимое.
///
/// [enableDrag] - индикатор того, можно ли скрыть попап жестом свайпа снизу
/// вверх. Его включение также отображает в верхней части попапа горизонтальную
/// полоску-индикатор доступности этого жеста. По умолчанию, true.
/// !!! Даже если enableDrag == false, на Android можно закрыть попап нажатием кнопки "назад"
///
/// [isDismissible] - индикатор того, можно ли скрыть попап тапом по пустому
/// месту на экране снаружи попапа. По умолчанию, true.
///
/// [isViewInsetsPaddingEnabled] - включает отступ, который нужен, чтобы попап
/// показывался над клавиатурой
///
/// [isScrollingEnabled] - включает скролл внутри попапа, который нужен, если
/// содержимое попапа может не умещаться на экране
Future<T?> showALModalPopup<T>({
  required BuildContext context,
  required Widget body,
  Color? backgroundColor,
  Color? barrierColor,
  bool enableDrag = true,
  bool isDismissible = true,
  double headerHeight = 40.0,
  double bottomOffset = 16.0,
  double topMargin = 0.0,
  bool isViewInsetsPaddingEnabled = false,
  bool isScrollingEnabled = false,
}) {
  return showModalBottomSheet(
    context: context,
    isScrollControlled: true,
    isDismissible: isDismissible,
    enableDrag: enableDrag,
    backgroundColor: Colors.transparent,
    barrierColor: barrierColor ?? context.colors.coolGrey,
    builder: (context) {
      return _ALModalBottomSheet(
        hasDragIndicator: enableDrag,
        backgroundColor: backgroundColor ?? context.colors.whiteElevation,
        headerHeight: headerHeight,
        bottomOffset: bottomOffset,
        topMargin: topMargin,
        body: body,
        isViewInsetsPaddingEnabled: isViewInsetsPaddingEnabled,
        isScrollingEnabled: isScrollingEnabled,
      );
    },
  );
}

class _ALModalBottomSheet extends StatelessWidget {
  final bool hasDragIndicator;
  final Color backgroundColor;
  final double headerHeight;
  final double bottomOffset;
  final Widget body;

  /// Добавляет нижний отступ, чтобы попап не перекрывался клавиатурой
  final bool isViewInsetsPaddingEnabled;

  /// Оборачивает содержимое в SingleChildScrollView, чтобы попап можно было скроллить,
  /// если он не умещается
  final bool isScrollingEnabled;

  /// Отступ от верхнего края экрана, чтобы ограничить высоту попапа
  final double topMargin;

  const _ALModalBottomSheet({
    required this.hasDragIndicator,
    required this.backgroundColor,
    required this.headerHeight,
    required this.bottomOffset,
    required this.topMargin,
    required this.body,
    required this.isViewInsetsPaddingEnabled,
    required this.isScrollingEnabled,
  });

  @override
  Widget build(BuildContext context) {
    final screenHeight = MediaQuery.of(context).size.height;

    return ConstrainedBox(
        constraints: BoxConstraints(maxHeight: screenHeight * 0.75),
        child: _buildPopup(context)
    );
  }

  Widget _buildPopup(BuildContext context) {
    return Container(
      /// Отступ нужен, чтобы popup показывался над клавиатурой
      padding:
          isViewInsetsPaddingEnabled ? MediaQuery.of(context).viewInsets : null,
      margin: EdgeInsets.only(top: topMargin),
      decoration: _getRoundCornersDecoration(context),
      clipBehavior: Clip.hardEdge,
      child: SafeArea(
        minimum: EdgeInsets.only(bottom: bottomOffset),
        child: Stack(
          children: [
            Padding(
              padding: EdgeInsets.only(top: headerHeight),
              child: isScrollingEnabled
                  ? SingleChildScrollView(child: body,)
                  : body,
            ),
            _buildHeader(context),
          ],
        ),
      ),
    );
  }

  BoxDecoration _getRoundCornersDecoration(BuildContext context) {
    return BoxDecoration(
      color: backgroundColor,
      borderRadius: const BorderRadius.vertical(top: Radius.circular(16)),
    );
  }

  Widget _buildHeader(BuildContext context) {
    if (hasDragIndicator) {
      return Container(
        decoration: _getRoundCornersDecoration(context),
        height: headerHeight,
        child: Container(
          alignment: Alignment.topCenter,
          padding: const EdgeInsets.only(top: 8),
          child: _buildDragIndicator(context),
        ),
      );
    } else {
      return const SizedBox.shrink();
    }
  }

  Widget _buildDragIndicator(BuildContext context) {
    return Container(
      height: 4,
      width: 120,
      decoration: BoxDecoration(
        color: context.colors.coolGrey,
        borderRadius: BorderRadius.circular(4),
      ),
    );
  }
}
