import 'dart:io';

import 'package:flutter/material.dart';
import 'package:altv/framework/extensions/build_context_extension.dart';
import 'package:altv/framework/log/log.dart';
import 'package:altv/framework/ui/widgets/popup/al_alert_popup.dart';
import 'package:altv/framework/ui/widgets/input/al_input_text.dart';
import 'package:altv/framework/ui/widgets/web_view/al_web_view_controller.dart';
import 'package:altv/framework/ui/widgets/web_view/al_web_view_action.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';

class ALWebView extends StatefulWidget {
  final ALWebViewAction Function({
    required Uri? currentUrl,
    required Uri newUrl,
  }) urlActionResolver;

  final Function(Uri url) onOpenUrlInBrowser;

  final VoidCallback? onLoadStop;
  final void Function(String? title)? onTitleChanged;
  final VoidCallback? onEnterFullScreen;
  final VoidCallback? onExitFullScreen;

  final Uri? initialUrl;
  final String? initialHtml;
  final ALWebViewController? controller;
  final Map<String, String>? cookies;
  final String predefinedHttpAuthUsername;
  final String predefinedHttpAuthPassword;

  final bool isScrollBarEnabled;
  final int? textZoom;

  const ALWebView({
    Key? key,
    required this.onOpenUrlInBrowser,
    this.initialUrl,
    this.initialHtml,
    this.urlActionResolver = _defaultUrlActionResolver,
    this.onLoadStop,
    this.onTitleChanged,
    this.onEnterFullScreen,
    this.onExitFullScreen,
    this.controller,
    this.cookies,
    this.predefinedHttpAuthUsername = "",
    this.predefinedHttpAuthPassword = "",
    this.isScrollBarEnabled = true,
    this.textZoom,
  }) : super(key: key);

  WebUri? get _initialWebUrl => initialUrl != null ? WebUri.uri(initialUrl!) : null;

  static ALWebViewAction _defaultUrlActionResolver({
    required Uri? currentUrl,
    required Uri newUrl,
  }) {
    if (currentUrl?.host == newUrl.host) {
      return ALWebViewAction.openInWebView;
    } else {
      return ALWebViewAction.openInBrowser;
    }
  }

  @override
  State<ALWebView> createState() => _ALWebViewState();
}

class _ALWebViewState extends State<ALWebView> {
  static final blankWebUrl = WebUri.uri(Uri.parse("about:blank"));

  InAppWebViewController? _webViewController;

  final _passwordFocusNode = FocusNode();
  final _passwordController = TextEditingController();
  final _usernameFocusNode = FocusNode();
  final _usernameController = TextEditingController();

  var _isInProgress = false;
  var _isWebViewCreated = false;
  var _progress = 0;
  var _isErrorMessageVisible = false;
  var _isCookiesApplied = false;
  var _isHttpAuthPopupShown = false;

  Future<HttpAuthResponse>? httpAuthResponse;

  @override
  void initState() {
    super.initState();
    // iOS 11 нужно, чтобы перед установкой куков сначала показался WebView.
    // Т.ч. сразу показываем WebView (вызывается build), затем сразу
    // подкладываем куки, и перезагружаем страницу
    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (_isCookiesApplied) {
        return;
      }
      _applyCookies().catchError((e) {
        debugPrint(e);
      }).whenComplete(() {
        _isCookiesApplied = true;
        setState(() {
          _webViewController?.reload();
        });
      });
    });
  }

  Future<void> _applyCookies() async {
    if (widget.cookies?.isNotEmpty != true) {
      return;
    }
    final cookieManager = CookieManager.instance();
    final keys = widget.cookies?.keys ?? [];
    for (final name in keys) {
      await cookieManager.setCookie(
        url: widget._initialWebUrl!,
        name: name,
        value: widget.cookies?[name] ?? "",
        isSecure: true,
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        _buildWebView(),
        // Пока WebView не проинициализировалась, закрываем ее белым экраном
        if (!_isWebViewCreated) Container(color: context.colors.white),
        if (_isInProgress) _buildLoadIndicator(),
      ],
    );
  }

  Widget _buildWebView() {
    return InAppWebView(
      initialData: widget.initialHtml != null
          ? InAppWebViewInitialData(data: widget.initialHtml!)
          : null,
      initialUrlRequest: URLRequest(url: widget._initialWebUrl ?? blankWebUrl),
      initialSettings: _getCurrentWebViewSettings(),
      shouldOverrideUrlLoading: _shouldOverrideUrlLoading,
      onEnterFullscreen: (controller) => widget.onEnterFullScreen?.call(),
      onExitFullscreen: (controller) => widget.onExitFullScreen?.call(),
      onWebViewCreated: _onWebViewCreated,
      onLoadStart: _onLoadStart,
      onProgressChanged: _onProgressChanged,
      onLoadStop: (controller, url) {
        _isInProgress = false;
        widget.onLoadStop?.call();
      },
      onReceivedError: _onReceivedError,
      onReceivedHttpError: _onReceivedHttpError,
      onTitleChanged: (controller, title) => widget.onTitleChanged?.call(title),
      onReceivedHttpAuthRequest: _onReceivedHttpAuthRequest,
    );
  }

  InAppWebViewSettings _getCurrentWebViewSettings() {
    return InAppWebViewSettings(
        textZoom: widget.textZoom,
        supportZoom: false,
        useShouldOverrideUrlLoading: true,
        transparentBackground: true,
        verticalScrollBarEnabled: widget.isScrollBarEnabled,
        horizontalScrollBarEnabled: widget.isScrollBarEnabled,
        forceDark: context.isDarkMode ? ForceDark.ON : ForceDark.OFF,
        forceDarkStrategy: ForceDarkStrategy.WEB_THEME_DARKENING_ONLY,
        useHybridComposition: true);
  }

  Future<NavigationActionPolicy> _shouldOverrideUrlLoading(
      InAppWebViewController controller, NavigationAction request) async {
    if (!Platform.isAndroid &&
        request.navigationType != NavigationType.LINK_ACTIVATED) {
      return NavigationActionPolicy.ALLOW;
    }

    var currentUrl = await _webViewController?.getUrl();
    if (currentUrl == null || currentUrl == blankWebUrl) {
      currentUrl = widget._initialWebUrl ?? blankWebUrl;
    }
    final newUrl = request.request.url;
    if (newUrl == null) {
      return NavigationActionPolicy.CANCEL;
    }

    final webViewAction = widget.urlActionResolver(
      currentUrl: currentUrl,
      newUrl: newUrl,
    );

    switch (webViewAction) {
      case ALWebViewAction.openInWebView:
        return NavigationActionPolicy.ALLOW;
      case ALWebViewAction.openInBrowser:
        widget.onOpenUrlInBrowser.call(newUrl);
        return NavigationActionPolicy.CANCEL;
      case ALWebViewAction.cancel:
        return NavigationActionPolicy.CANCEL;
    }
  }

  Future<void> _onWebViewCreated(InAppWebViewController controller) async {
    _webViewController = controller;
    widget.controller?.webViewController = controller;
    // После инициализации WebView иногда моргает, т.ч. немного ждем,
    // прежде чем сообщать об окончании инициализации
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (mounted) {
        setState(() {
          _isWebViewCreated = true;
        });
      }
    });
  }

  void _onLoadStart(InAppWebViewController controller, Uri? url) {
    setState(() {
      _isInProgress = true;
    });
  }

  void _onProgressChanged(InAppWebViewController controller, int progress) {
    setState(() {
      _progress = progress;
      if (_progress == 100) {
        _isInProgress = false;
      } else if (_progress > 0) {
        _isInProgress = true;
      }
    });
  }

  void _onReceivedError(
    InAppWebViewController controller,
    WebResourceRequest request,
    WebResourceError error,
  ) {
    final url = request.url;
    final code = error.type.toString();
    final message = error.description;

    Log.d('onLoadError: $url $code $message');
    setState(() {
      _isInProgress = false;
    });
    _showError();
  }

  void _onReceivedHttpError(
    InAppWebViewController controller,
    WebResourceRequest request,
    WebResourceResponse errorResponse,
  ) {
    final url = request.url;
    final statusCode = errorResponse.statusCode;
    final description = errorResponse.reasonPhrase;

    Log.d('onLoadHttpError $url, $statusCode $description');
    setState(() {
      _isInProgress = false;
    });
    _showError();
  }

  Future<HttpAuthResponse?> _onReceivedHttpAuthRequest(
    InAppWebViewController controller,
    URLAuthenticationChallenge challenge
  ) async {
    if (!_isHttpAuthPopupShown) {
      _isHttpAuthPopupShown = true;
      httpAuthResponse = _showAuthPopup();
    }
    if (httpAuthResponse == null) {
      return null;
    }
    HttpAuthResponse response = await httpAuthResponse!;
    _isHttpAuthPopupShown = false;
    return response;
  }

  Widget _buildLoadIndicator() {
    return Container(
      alignment: Alignment.topCenter,
      child: LinearProgressIndicator(
        value: _progress.toDouble() / 100,
        minHeight: 1.0,
        backgroundColor: context.colors.white,
      ),
    );
  }

  void _showError() {
    // Бывают странные ошибки загрузки при возврате по истории навигации.
    // В этом случае ошибку не показываем.
    if (_isErrorMessageVisible || _progress == 100) {
      return;
    }
    _isErrorMessageVisible = true;
    showALAlertPopup(
      context: context,
      title: "Не удалось загрузить страницу",
      text: null,
      buttons: [
        ALAlertPopupButton(
          text: "Закрыть",
          onPressed: () => _isErrorMessageVisible = false,
          isDismissing: true,
        ),
        ALAlertPopupButton(
          text: "Повторить",
          onPressed: () async {
            _isErrorMessageVisible = false;
            await _reloadPage();
          },
        )
      ],
    );
  }

  Future<void> _reloadPage() async {
    final currentUrl = await _webViewController?.getUrl();
    if (currentUrl == null && widget.initialUrl != null) {
      final request = URLRequest(url: widget._initialWebUrl);
      _webViewController?.loadUrl(urlRequest: request);
    } else {
      await _webViewController?.reload();
    }
  }

  Future<HttpAuthResponse> _showAuthPopup() async {
    late HttpAuthResponse response;
    _usernameController.text = widget.predefinedHttpAuthUsername;
    _passwordController.text = widget.predefinedHttpAuthPassword;
    await showALAlertPopup<HttpAuthResponse>(
      context: context,
      title: 'Авторизуйтесь',
      text: '',
      textAlign: TextAlign.left,
      isDismissible: false,
      additionalContent: _buildHttpAuthInputWidgets(),
      viewInsetsPaddingEnabled: true,
      buttons: [
        ALAlertPopupButton(
          text: 'Ок',
          isDismissing: true,
          onPressed: () {
            response = HttpAuthResponse(
                username: _usernameController.text,
                password: _passwordController.text,
                action: HttpAuthResponseAction.PROCEED);
          },
        ),
        ALAlertPopupButton(
          text: 'Отмена',
          isDismissing: true,
          onPressed: () {},
        ),
      ],
    );
    return response;
  }

  Widget _buildHttpAuthInputWidgets() {
    return Column(
      children: [
        ALInputText(
          title: "Введите имя пользователя",
          focusNode: _usernameFocusNode,
          controller: _usernameController,
        ),
        ALInputText(
          title: "Введите пароль",
          focusNode: _passwordFocusNode,
          controller: _passwordController,
        ),
      ],
    );
  }

  @override
  void dispose() {
    super.dispose();
    _passwordController.dispose();
    _passwordFocusNode.dispose();
    _usernameController.dispose();
    _usernameFocusNode.dispose();
  }
}

